Unit GetCmd;

(****************************************************************************
**                                                                         **
**  Parse command line and function file (if required)                     **
**                                                                         **
****************************************************************************)

{

  Important additions for next version:

    Allow flags to be set using a "NOoption" syntax in addition to the
      current "option=(YES/NO)" style.  Thus,  the parameter "NOWIREFRAME"
      would be interpreted in the same way as "WIREFRAME=NO".

}

Interface

Procedure GetCommandLine;

Implementation

Uses
  DOS,       {  }
  Graphics,  {  }
  FileIO,    {  }
  Print,     {  }
  Vectors,   { general floating-point I/O }
  ParsEval,  { parser/evaluator }
  QuadFrac,  { quadratic fractal operations }
  GDrivers,  { graphics drivers and mode/device parsing }
  FDecls;    { general global definitions }

{---------------------------------------------------------------------------}

Procedure GetCommandLine;
{
  check user's input parameters
}
Const
  FunctionFile : Boolean = False;  { function file available? }
  FuncFileName : String[128] = ''; { name of function file }
Var
  Error,                  { error number (from Parse) }
  Posn,                   { error posn in string (from Parse) }
  I            : Integer; { loop counter through parameters }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Function Exists(Path : String) : Boolean;
  {
    determine if file exists
  }
  Var
    F : File;
    B : Boolean;
  Begin
    Assign(F, Path);
    {$I-} Reset(F); {$I+}
    B := IOResult = 0;
    If B then
      Close(F);
    Exists := B
  End; { Function Exists }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Function Str2Float(S : String) : Float;
  {
    convert constant expression into numeric value.
    returns ConstError if input was a bad expression
  }
  Begin
    Str2Float := EvalConst(S)
  End; { Function Str2Float }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Function Str2Int(S : String) : Integer;
  {
    convert constant expression to integer
  }
  Var
    TempFloat : Float;
  Begin
    TempFloat := Str2Float(S);
    If (TempFloat >= -32768.0) and (TempFloat <= 32767.0) then
      Str2Int := Round(TempFloat)
    Else
      Abort('Out-of-range error in integer constant expression');
  End; { Function Str2Int }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Function StrUpCase(S : String) : String;
  {
    return upper case version of string
  }
  Var
    I : Integer;
  Begin
    For I := 1 to Length(S) do
      S[I] := UpCase(S[I]);
    StrUpCase := S
  End; { Function StrUpCase }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Function StripChars(S, Mask : String) : String;
  {
    strip leading characters from S that match Mask
    (and possibly some equals sign(s))
  }
  Var
    I : Integer;
  Begin

    { find end of mask matching area }
    I := 1;
    While (I <= Length(Mask)) and (S[I] = Mask[I]) do
      Inc(I);

    { remove mask area }
    Delete(S, 1, I-1);

    { strip equals sign(s) if available }
    While Pos('=', S) = 1 do
      Delete(S, 1, 1);

    { return result }
    StripChars := S

  End; { Function StripChars }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Function CheckAndSetString(S, Mask, TakeFrom : String;
                             Var DataVal : String) : Boolean;
  {
    return string contained in data
  }
  Var
    S1 : String;
    I  : Integer;

    Function Pad(N : Integer) : String;
    {
      return 2-digit number left padded with zeroes
    }
    Var
      S : String[20];
    Begin
      S := '00' + Int2Str(N);
      Pad := Copy(S, Length(S) - 1, 2)
    End; { Function Pad }

    Function TimeString : String;
    {
      return current system clock time in hours:minutes:seconds
    }
    Var
      Hour, Min, Sec, Sec100 : Word;
    Begin

      { read system clock }
      GetTime(Hour, Min, Sec, Sec100);

      { convert to string and return }
      If Hour = 0 then
        TimeString := '12:' + Pad(Min)
      Else If Hour <= 12 then
        TimeString := Pad(Hour) + ':' + Pad(Min) + 'a'
      Else
        TimeString := Pad(Hour - 12) + ':' + Pad(Min) + 'p'

    End; { Function TimeString }

    Function DateString0 : String;
    {
      return current system date in month/day/year
    }
    Var
      Year, Month, Day, DayOfWeek : Word;
    Begin

      { read system clock }
      GetDate(Year, Month, Day, DayOfWeek);

      { convert to string and return }
      DateString0 := Pad(Month) + '/' + Pad(Day) + '/' + Pad(Year)

    End; { Function DateString0 }

    Function DateString1 : String;
    {
      return date in DayOfWeek, Month, Year
    }
    Const
      WeekDays : Array[0..6] of String[9] = (
        'Sunday', 'Monday', 'Tuesday', 'Wednesday',
        'Thursday', 'Friday', 'Saturday');
      Months : Array[1..12] of String[9] = (
        'January', 'February', 'March', 'April', 'May', 'June',
        'July', 'August', 'September', 'October', 'November', 'December');
    Var
      Year, Month, Day, DayOfWeek : Word;
    Begin

      { read system clock }
      GetDate(Year, Month, Day, DayOfWeek);

      { convert to string and return }
      DateString1 := WeekDays[DayOfWeek] + ', ' +
                     Months[Month] + ' ' + Int2Str(Day) + ', ' +
                     Int2Str(Year)

    End; { Function DateString1 }

  Begin

    { assume we haven't found a string yet }
    CheckAndSetString := False;

    { check for mask }
    If Pos(Mask, S) = 1 then begin

      { found mask }
      CheckAndSetString := True;

      { set correct value }
      S := TakeFrom;

      { remove parameter data }
      I := 1;
      While (I <= Length(Mask)) and (UpCase(S[I]) = UpCase(Mask[I])) do
        Inc(I);
      Delete(S, 1, I - 1);

      { strip leading spaces and equals signs }
      While (S[1] = ' ') or (S[1] = '=') do
        Delete(S, 1, 1);

      { strip trailing spaces }
      While S[Length(S)] = ' ' do
        Dec(S[0]);

      { process parameters }
      While Pos('__', S) > 0 do begin

        { remember where parameter went }
        I := Pos('__', S);

        { remove lead-in }
        Delete(S, I, 2);

        { extract parameter }
        S1 := '';
        Repeat
          S1 := S1 + StrUpCase(Copy(S, I, 1));
          Delete(S, I, 1)
        Until Copy(S, I, 1) = '_';

        { remove trailer }
        Delete(S, I, 2);

        { check parameter and act upon it }
        If S1 = 'VER' then
          Insert(Version, S, I)
        Else If S1 = 'TIME' then
          Insert(TimeString, S, I)
        Else If S1 = 'DATE' then
          Insert(DateString0, S, I)
        Else If S1 = 'XDATE' then
          Insert(DateString1, S, I)

      End; { While }

      { set up string }
      DataVal := S

    End { If }

  End; { Function CheckAndSetString }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Procedure CheckParm(Parm : String);
  {
    check given string and set parameters accordingly
  }
  Var
    I : Integer;               { scratch                                    }
    S : String;                { scratch                                    }
    NumParms,                  { # parameters found on string               }
    TempInt      : Integer;    { scratch for numeric conversions            }
    Parms        : Array[1..32] of Float;      { parameters found on string }
    ParmText     : Array[1..32] of String[64];

    TempFloat    : Float;      { scratch }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Function GetParms(S, Mask : String) : Integer;
    {
      Get constant parameters from input string.
      Returns the parameter values in Parms[] and parameter text in ParmText[].
      Returns the number of parameters as the function return value and
      in NumParms.
    }
    Var
      S1 : String;
      TempFloat : Float;
    Begin

      { strip parm from front end and ensure the system sees at least 1 parm }
      S := StripChars(S, Mask) + ',';

      { no parms found yet }
      NumParms := 0;

      { go until no parms left or no room left in table }
      While (Pos(',', S) > 0) and
            (NumParms <= SizeOf(Parms) div SizeOf(Parms[1])) do begin

        { extract this parm }
        S1 := Copy(S, 1, Pos(',', S)-1);

        { remove it from the input }
        Delete(S, 1, Pos(',', S));

        { convert it to a number }
        TempFloat := Str2Float(S1);

        { if it was a good number (or expression) }
        If TempFloat <> ConstError then begin

          { add it to the list }
          Inc(NumParms);
          Parms[NumParms] := TempFloat;
          ParmText[NumParms] := S1;

        End; { If }

      End; { While }

      GetParms := NumParms;

    End; { Procedure GetParms }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Function GetSParms(S, Mask : String) : Integer;
    {
      Get string parameters from input string (no parsing or other operations).
      Returns the parameter text in ParmText[].
      Returns the number of parameters as the function return value and
      in NumParms.
    }
    Var
      S1 : String;
    Begin

      { strip parm from front end and ensure the system sees at least 1 parm }
      S := StripChars(S, Mask) + ',';

      { no parms found yet }
      NumParms := 0;

      { go until no parms left or no room left in table }
      While (Pos(',', S) > 0) and
            (NumParms <= SizeOf(Parms) div SizeOf(Parms[1])) do begin

        { extract this parm }
        S1 := Copy(S, 1, Pos(',', S)-1);

        { remove it from the input }
        Delete(S, 1, Pos(',', S));

        { add it to the list }
        Inc(NumParms);
        Parms[NumParms] := TempFloat;
        ParmText[NumParms] := S1;

      End; { While }

      GetSParms := NumParms;

    End; { Procedure GetSParms }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Function CheckAndSetFloat(S, Mask : String; Var DataVal : Float) : Boolean;
    {
      check if parm is to be adjusted and do so if required
    }
    Begin

      { assume something was wrong }
      CheckAndSetFloat := False;

      { if it's this parameter }
      If Pos(Mask, S) = 1 then begin

        If GetParms(S, Mask) = 1 then begin

          { return data value }
          DataVal := Parms[1];

          { return found flag }
          CheckAndSetFloat := True

        End { If }

      End { If }

    End; { Procedure CheckAndSetFloat }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Function CheckAndSetFlag(S, Mask : String; Var Flag : Boolean) : Boolean;
    {
      check if parm is to be adjusted and do so if required
    }
    Begin

      { if it's this parameter }
      If Pos(Mask, S) = 1 then begin

        { set flag }
        Flag := Copy(StripChars(S, Mask), 1, 1) <> 'N';

        { return found flag }
        CheckAndSetFlag := True

      End { If }
      Else
        CheckAndSetFlag := False

    End; { Procedure CheckAndSetFlag }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Procedure GiveHelp;
    {
      give minor help to user and dump out of program
    }
    Begin
      Writeln;
      Writeln('FPLOT [options] expression');
      Writeln;
      Writeln('RANGE=n              set +/- evaluation range about 0, 0');
      Writeln('CORNERS=x1, y1, x2, y2  set corners of evaluation grid');
      Writeln('VPOINT=x, y, z         set X, Y, Z viewpoint');
      Writeln('GRIDSIZE=i           set evaluation grid size');
      Writeln('WIREFRAME=c          draw wireframe in given color');
      Writeln('DITHER=f             simulate colors on mono screen');
      Writeln('SHADED=f             draw shading on VGA screen');
      Writeln('COLOR=c              set drawing color');
      Writeln('DELAY=n              when done, delay n seconds then exit');
      Writeln;
      Writeln('IMPORTANT: enter NO spaces in the expression.');
      Writeln;
      Writeln('See program documentation for a full list ' +
              'of available options.');
      Halt;
    End; { Procedure GiveHelp }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Function CheckFileOper(S : String) : Boolean;
    {
      check for file operation and make sure parameters are as required
    }
    Var
      TypeString : String;   { save type }
      I          : Integer;  { search index }
    Begin

      { check for load command }
      If Pos('LOAD', S) = 1 then begin

        { set operation type }
        FileOper := Load;

        { get file name }
        S := StripChars(S, 'LOAD');

        { make sure desired file exists }
        If not Exists(S) then
          Abort('Unable to locate file ' + S);

        { make sure data file name makes it out alive }
        DataFileName := S;

        { if we made it to here, the file data must be OK }
        CheckFileOper := True

      End { If }

      { check for save command }
      Else If Pos('SAVE', S) = 1 then begin

        { set file operation }
        FileOper := Save;

        { get type and name }
        S := StripChars(S, 'SAVE');

        { make sure there is a separator }
        If Pos(',', S) > 1 then begin

          { remove file type identifier }
          TypeString := Copy(S, 1, Pos(',', S)-1);

          { S should contain the file name after this command }
          Delete(S, 1, Pos(',', S));

          { try to locate specified file type }
          I := SurfData;
          While (I <= SurfGIF) and (FileIDNames[I] <> TypeString) do
            Inc(I);

          { if it was found }
          If I <= SurfGIF then begin

            { set file type }
            FileType := I;

            { make sure data file name makes it out alive }
            DataFileName := S;

            { if we made it to here, the file data must be OK }
            CheckFileOper := True

          End { If }

        End { If }

      End { Else If }

      { wasn't load or save }
      Else
        CheckFileOper := False

    End; { Function CheckFileOper }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Procedure ParseLighting(Parm : String);
    {
      check lighting arrangements
    }
    Var
      S : String;
      LightNum : Integer;
    Begin

      { determine which light to modify }
      S := Copy(Parm, 1, Pos(']', Parm)-1);
      LightNum := Str2Int(S);

      { if valid light ID }
      If LightNum in [1..MaxLights] then begin

        { strip light number }
        Delete(Parm, 1, Pos(']', Parm));

        { strip equals sign }
        While Parm[1] = '=' do
          Delete(Parm, 1, 1);

        { remove sub-parameter }
        S := Copy(Parm, 1, Pos('=', Parm));
        Delete(Parm, 1, Pos('=', Parm));

        { get attached parameters }
        If GetParms(Parm, '') = 0 then ;

        { reduce sub-parameter to workable string }
        While S[Length(S)] = '=' do
          Dec(S[0]);

        { set light position and intensity as requested }
        If NumParms >= 3 then

          With Lights[LightNum] do begin

            { make sure that the light is known as being in use }
            InUse := True;

            { tell what kind of light this is }
            IsSpot := False;

            { set light location in correct coord system }
            If Pos('AZE', S) = 1 then begin  { polar }
              PolarToRect(Parms[1], Parms[2], Parms[3]);
              Posn.X := Parms[1];
              Posn.Y := Parms[2];
              Posn.Z := Parms[3]
            End { If }
            Else begin                 { rectangular }
              Posn.X := Parms[1];
              Posn.Y := Parms[2];
              Posn.Z := Parms[3]
            End; { If }

            { set light brightness if given }
            If NumParms = 4 then
              Bright[RColor] := Parms[4]

            { use all color components if given }
            Else If NumParms = 6 then begin
              Bright[RColor] := Parms[4];
              Bright[GColor] := Parms[5];
              Bright[BColor] := Parms[6];
              MaxComponent := BColor
            End { If }

          End { With }

      End { If }

    End; { Procedure ParseLighting }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Procedure ParseSpotLighting(Parm : String);
    {
      check lighting arrangements
    }
    Var
      CParm    : Integer;
      S        : String;
      LightNum : Integer;
      One      : Float;
    Begin

      { determine which light to modify }
      S := Copy(Parm, Pos('[', Parm)+1, Pos(']', Parm) - Pos('[', Parm) - 1);
      LightNum := Str2Int(S);

      { if valid light ID }
      If LightNum in [1..MaxLights] then begin

        { strip light number }
        Delete(Parm, 1, Pos(']', Parm));

        { strip equals sign }
        While Parm[1] = '=' do
          Delete(Parm, 1, 1);

        { remove sub-parameter }
        S := Copy(Parm, 1, Pos('=', Parm));
        Delete(Parm, 1, Pos('=', Parm));

        { get attached parameters }
        If GetParms(Parm, '') = 0 then ;

        { reduce sub-parameter to workable string }
        While S[Length(S)] = '=' do
          Dec(S[0]);

        { set light position and intensity as requested }
        If NumParms >= 6 then

          With Lights[LightNum] do begin

            { make sure that the light is known as being in use }
            InUse := True;

            { set what kind of light this is }
            IsSpot := True;

            { set light location in correct coord system }
            If (Pos('AZE', S) = 1)or(Pos('XYZ', S) = 1)or(S = '') then begin

              { convert polar coords to XYZ coords }
              If Pos('AZE', S) = 1 then
                PolarToRect(Parms[1], Parms[2], Parms[3]);

              { set position }
              Posn.X := Parms[1];
              Posn.Y := Parms[2];
              Posn.Z := Parms[3];

              { get direction (unit vector) }
              One := 1.0;
              PolarToRect(Parms[4], Parms[5], One);
              Direction.i := Parms[4];
              Direction.j := Parms[5];
              Direction.k := One;

              { start after parm list }
              CParm := 6

            End { If }

            Else If Pos('CAM', S) = 1 then begin  { camera model }

              { set light position }
              Posn.X := Parms[1];
              Posn.Y := Parms[2];
              Posn.Z := Parms[3];

              { got the lookfrom and the lookto parms, get the direction }
              MakeUnitVector(Parms[4] - Posn.X,
                             Parms[5] - Posn.Y,
                             Parms[6] - Posn.Z, Direction);

              { start after parm list }
              CParm := 7

            End; { Else If }

            { get beamwidth exponent }
            SpotBWExp := Parms[CParm];
            Inc(CParm);

            { get cone angle }
            CosConeAngle := Pi / 2;
            If CParm <= NumParms then begin
              Parms[CParm] := Angle(Parms[CParm]);
              If (Parms[CParm] >= 0) and (Parms[CParm] < Pi / 2) then
                CosConeAngle := cos(Parms[CParm]);
              Inc(CParm)
            End; { If }

            { set light brightness if given }
            If CParm = NumParms then
              Bright[RColor] := Parms[CParm]

            { use all color components if given }
            Else If CParm < NumParms then begin
              Bright[RColor] := Parms[CParm + 0];
              Bright[GColor] := Parms[CParm + 1];
              Bright[BColor] := Parms[CParm + 2];
              MaxComponent := BColor
            End { If }

          End { With }

      End { If }

    End; { Procedure ParseSpotLighting }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Procedure TranslateSolid(S : String);
    {
      translate solid color specs
    }
    Var
      Color1,
      Color2 : Integer;
    Begin

      { remove header }
      S := StripChars(S, 'SOLID');

      { make certain of separator }
      If Pos(',', S) > 0 then begin

        { get color parameters }
        Color1 := LookupColor(Copy(S, 1, Pos(',', S)-1));
        Color2 := LookupColor(Copy(S, Pos(',', S)+1, 255));

        { if parameters OK, then set colors }
        If (Color1 >= 0) and (Color2 >= 0) and (Color1 <> Color2) then begin
          WireFrameColor := Color1;
          FillColor := Color2
        End { If }

      End { If }

    End; { Procedure TranslateSolid }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Procedure ProcessFunctionFile(Name : String);
    {
      process function definition file
    }
    Var
      S : String;
      InFile : Text;
      ErrCode : Integer;

    Begin

      { try to open data file }
      Assign(InFile, Name);
      {$I-} Reset(InFile); {$I+}

      { check system's error }
      ErrCode := IOResult;

      { bomb if error }
      If ErrCode <> 0 then
        Abort('Error opening file ' + Name + ' code = ' + Int2Str(ErrCode));

      { read all lines in the data file }
      While not EOF(InFile) do begin

        { get a line }
        Readln(InFile, S);

        { strip possible comment }
        While Pos(';', S) > 0 do
          Delete(S, Pos(';', S), 255);

        { strip leading spaces }
        While Pos(' ', S) = 1 do
          Delete(S, 1, 1);

        { process data line if there's anything left }
        If S <> '' then
          CheckParm(S)

      End; { While }

      { done, close file }
      Close(InFile)

    End; { Procedure ProcessFunctionFile }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

    Function CheckBareCmdFile(Name : String) : Boolean;
    {
      Check if the string passed in is a filename.
    }
    Var
      DirInfo: SearchRec;
    Begin

      FindFirst(Name, Archive, DirInfo);
      If DosError = 0 then begin
        ProcessFunctionFile(Name);
        CheckBareCmdFile := True;
      End { If }
      Else begin
        FindFirst(Name + '.FNC', Archive, DirInfo);
        If DosError = 0 then begin
          ProcessFunctionFile(Name + '.FNC');
          CheckBareCmdFile := True;
        End { If }
        Else
          CheckBareCmdFile := False;

      End; { Else }

    End; { Procedure CheckBareCmdFile }

    {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Begin

    { convert input parameter to scratch var in upper case }
    S := StrUpCase(Parm);

    { strip leading slashes }
    While Pos('/', S) = 1 do
      Delete(S, 1, 1);

    { strip out spaces }
    While Pos(' ', S) > 0 do
      Delete(S, Pos(' ', S), 1);


    (* COMMAND CHECKING STARTS HERE *)

    { check for help }
    If (Pos('?', S) = 1) or (Pos('HELP', S) = 1) then
      GiveHelp


    { adjust grid size as required }
    Else If Pos('GRID', S) = 1 then begin
      If GetParms(S, 'GRIDSIZE') = 1 then
        If (Parms[1] >= MinGridSize) and (Parms[1] <= MaxGridSize) then begin
          XGridSize := Round(Parms[1]);
          YGridSize := Round(Parms[1])
        End { If }
    End { If }

    { adjust X grid size as required }
    Else If Pos('XGRID', S) = 1 then begin
      If GetParms(S, 'XGRIDSIZE') = 1 then
        If (Parms[1] >= MinGridSize) and (Parms[1] <= MaxGridSize) then
          XGridSize := Round(Parms[1]);
    End { If }


    { adjust Y grid size as required }
    Else If Pos('YGRID', S) = 1 then begin
      If GetParms(S, 'YGRIDSIZE') = 1 then
        If (Parms[1] >= MinGridSize) and (Parms[1] <= MaxGridSize) then
          YGridSize := Round(Parms[1]);
    End { If }

    Else If Pos('COPIES', S) = 1 then begin
      If (GetParms(S, 'COPIES') = 1) and (Parms[1] > 1) then
        NCopies := Round(TempInt)
    End { If }


    { use degrees for angles? }
    Else If Pos('DEG', S) = 1 then
      Radian := not (Copy(StripChars(S, 'DEGREES'), 1, 1) <> 'N')


    { use radians for angles? }
    Else If Pos('RAD', S) = 1 then
      Radian := Copy(StripChars(S, 'RADIANS'), 1, 1) <> 'N'


    { use circular grid? }
    Else If CheckAndSetFlag(S, 'CIRCULAR', Circular) then


    { use inverted halftones? }
    Else If CheckAndSetFlag(S, 'INVDRAW', InverseHalf) then


    { use floating Z focus value? }
    Else If CheckAndSetFlag(S, 'FLOATZ', FloatZ) then


    { change ambient lighting? }
    Else If Pos('AMBIENT', S) = 1 then begin

      If (GetParms(S, 'AMBIENT') = 1) or (NumParms = 3) then begin
        AmbientLight[RColor] := Parms[1];
        If NumParms = 3 then begin
          AmbientLight[GColor] := Parms[2];
          AmbientLight[BColor] := Parms[3];
          MaxComponent := BColor
        End { If }
      End { If }

    End { Else If }


    { change surface translucency? }
    Else If Pos('TRANSL', S) = 1 then begin
      If (GetParms(S, 'TRANSLUCENCY') = 1) or (NumParms = 3) then begin
        Translucency[RColor] := Parms[1];
        If NumParms = 3 then begin
          Translucency[GColor] := Parms[2];
          Translucency[BColor] := Parms[3];
          MaxComponent := BColor
        End { If }
      End { If }
    End { Else If }


    { seed random # generator with random number? }
    Else If Pos('RANDOMIZE', S) = 1 then
      Randomize

    { seed random # generator with specific number? }
    Else If Pos('RANDSEED', S) = 1 then begin
      If GetParms(S, 'RANDSEED') = 1 then
        RandSeed := Round(Parms[1]);
    End { If }

    { draw wireframe? }
    Else If Pos('WIRE', S) = 1 then begin

      { we know we need to draw the wireframe }
      WireFrame := True;

      { check for color name }
      TempInt := LookUpColor(StripChars(S, 'WIREFRAME'));

      { if color was valid, set the color }
      If TempInt >= 0 then
        WireFrameColor := TempInt

    End { Else If }


    { fill color for wireframe? }
    Else If Pos('FILL', S) = 1 then begin

      { check for color name }
      TempInt := LookUpColor(StripChars(S, 'FILLCOLOR'));

      { if color was valid, set the color }
      If TempInt >= 0 then
        FillColor := TempInt

    End { Else If }


    { shade interiors? }
    Else If Pos('SHADE', S) = 1 then
      Shaded := Copy(StripChars(S, 'SHADED'), 1, 1) <> 'N'


    { dither interiors? }
    Else If Pos('DITHE', S) = 1 then
      Dithered := Copy(StripChars(S, 'DITHERED'), 1, 1) <> 'N'


    { activate timer? }
    Else If CheckAndSetFlag(S, 'TIMER', TimerOn) then


    { show range values? }
    Else If CheckAndSetFlag(S, 'SHOWRANGE', ShowRange) then


    { show function values? }
    Else If CheckAndSetFlag(S, 'SHOWFUNC', ShowFunc) then

    { plasma exponent? }
    Else If CheckAndSetFloat(S, 'PLASMAEXP', PosPlasmaExponent) then begin
      If PosPlasmaExponent <> 0 then
        NegPlasmaExponent := 1.0 / PosPlasmaExponent;
    End { Else If }
    Else If CheckAndSetFloat(S, 'PLASMAPOSEXP', PosPlasmaExponent) then
    Else If CheckAndSetFloat(S, 'PLASMANEGEXP', PosPlasmaExponent) then 

    { smooth plasma surface? }
    Else If CheckAndSetFloat(S, 'PLASMASMOOTH', PlasmaSmooth) then

    { plasma surface? }
    Else If CheckAndSetFlag(S, 'PLASMA', DoPlasma) then
      PreCompute := True

    { check for max iteration values }
    Else If Pos('MAXITER', S) = 1 then begin
      If GetParms(S, 'MAXITER') = 1 then
        If (Parms[1] > 0) and (Parms[1] <= AbsMaxIterations) then
          Maxiterations := Round(Parms[1])
    End { Else If }


    { check for radius value }
    Else If CheckAndSetFloat('MAXRADIUS', S, MaxRadius) then
      MaxRadius := Sqr(MaxRadius)


    { check for julia set parameter (complex number) }
    Else If Pos('JULIAPARM', S) = 1 then begin
      If GetParms(S, 'JULIAPARM') = 1 then begin
        RealJuliaParm := Parms[1];
        ImagJuliaParm := 0.0
      End { If }
      Else If NumParms = 2 then begin
        RealJuliaParm := Parms[1];
        ImagJuliaParm := Parms[2]
      End { Else }
    End { Else If }


    { show axes? }
    Else If CheckAndSetFlag(S, 'SHOWAXES', ShowAxes) then


    { check for math errors? }
    Else If CheckAndSetFlag(S, 'CHECKMATH', MathCheck) then


    { what math error value should be used? }
    Else If CheckAndSetFloat(S, 'MATHERRVALUE', MathErrValue) then


    { check for printer port spec (trimmed because DOS can't use long names) }
    Else If Pos('PRINTERPORT', S) = 1 then
      PrintFileName := Copy(StripChars(S, 'PRINTERPORT'), 1, 120)

    { same as above, different name }
    Else If Pos('PRINTERFILE', S) = 1 then
      PrintFileName := Copy(StripChars(S, 'PRINTERFILE'), 1, 120)

    { print result? }
    Else If CheckAndSetFlag(S, 'PRINT', PrintImage) then

    { check for inverse (white on black) printing }
    Else If Pos('INVERSE', S) = 1 then
      PrintMode := (PrintMode and not Inverse) or
          (Inverse * Ord(Copy(StripChars(S, 'INVERSE'), 1, 1) <> 'N'))

    { check for PostScript screen parameters }
    Else If Pos('SCREEN', S) = 1 then begin

      { Get three parms - screen #, screen frequency, screen angle }
      If GetParms(S, 'SCREEN') = 3 then begin
        If (Parms[1] > 0) and (Parms[1] < NumScreens) then
          ScreenNum := Trunc(Parms[1]);
        If (Parms[2] > 10) and (Parms[2] < 500) then
          ScreenFreq := Parms[2];
        ScreenAngle := Parms[3]
      End; { If }

    End { Else If }

    { check for PostScript screen parameters }
    Else If Pos('LINEWIDTH', S) = 1 then begin
      If GetParms(S, 'LINEWIDTH') = 1 then
        If (Parms[1] >= 0) and (Parms[1] < 5) then
          PSLineWidth := Parms[1]
    End { Else If }


    { check for forced video mode }
    Else If Pos('GRAPHM', S) = 1 then
      GraphMode := ParseGraphMode(StripChars(S, 'GRAPHMODE'))

    { check for forced video device }
    Else If Pos('GRAPHD', S) = 1 then
      GraphDevice := ParseGraphDevice(StripChars(S, 'GRAPHDEVICE'))

    { check for function definition file }
    Else If Pos('FUNCTIONFILE', S) = 1 then
      ProcessFunctionFile(StripChars(S, 'FUNCTIONFILE'))
    Else If Pos('FUNCFILE', S) = 1 then
      ProcessFunctionFile(StripChars(S, 'FUNCFILE'))


    { turn off specular component }
    Else If CheckAndSetFlag(S, 'SPECULAR', USeSpec) then


    { check for shininess parameter }
    Else If Pos('SHINY', S) = 1 then begin

      { we'll be adding the shininess regardless of any further values }
      UseSpec := True;

      { if only one parameter, get value }
      If GetParms(S, 'SHINY') = 1 then
        If (Parms[1] > 0) and (Parms[1] < 255) then
          PhongExp := Round(Parms[1])

    End { Else If }


    { check adjustments to specular and diffuse constants }
    Else If CheckAndSetFloat(S, 'SPECCONST', SpecMulConst) then
    Else If CheckAndSetFloat(S, 'DIFFCONST', DiffMulConst) then


    { check for walls }
    Else If Pos('WALL', S) = 1 then begin

      If GetParms(S, 'WALL') = 2 then begin
        WallBase := Parms[1];
        WallHeight := Abs(Parms[2])
      End { If }

    End { Else }


    { check for discontinuity specification }
    Else If CheckAndSetFloat(S, 'DISCONTINUITY', Discontinuity) then
      Discontinuity := Abs(Discontinuity)


    { check for title string }
    Else If CheckAndSetString(S, 'TITLE', Parm, TitleString) then

    { check for text string }
    Else If CheckAndSetFlag(S, 'TEXTOFF', DrawInfo) then

    { check for text string }
    Else If CheckAndSetString(S, 'TEXTSTR', Parm, UserText) then

    { check for axis labels }
    Else If CheckAndSetString(S, 'XAXISLABEL', Parm, XAxisLabel) then
    Else If CheckAndSetString(S, 'YAXISLABEL', Parm, YAxisLabel) then
    Else If CheckAndSetString(S, 'ZAXISLABEL', Parm, ZAxisLabel) then


    { check for text position }
    Else If Pos('TEXTPOS', S) = 1 then begin

      If GetParms(S, 'TEXTPOS') >= 3 then begin
        UserTextXPos := Parms[1];
        UserTextYPos := Parms[2];
        UserTextZPos := Parms[3]
      End; { If }

      If NumParms = 6 then begin
        UserTextXRot := Angle(Parms[4]);
        UserTextYRot := Angle(Parms[5]);
        UserTextZRot := Angle(Parms[6])
      End; { If }

    End { Else If }


    { check for font size }
    Else If CheckAndSetFloat(S, 'FONTSIZE', FontSize) then


    { check for color setup }
    Else If Pos('COLOR', S) = 1 then begin

      { translate color value }
      TempInt := LookupColor(StripChars(S, 'COLOR'));

      { if valid }
      If TempInt > 0 then
        WireFrameColor := TempInt

    End { Else If }


    { check for background color }
    Else If Pos('BACK', S) = 1 then begin

      { translate color value }
      TempInt := LookUpColor(StripChars(S, 'BACKGROUNDCOLOR'));

      { if valid color }
      If TempInt > 0 then
        BackGndColor := TempInt      { make sure low-intensity }

    End { Else If }


    { check for text color }
    Else If Pos('TEXTCOLOR', S) = 1 then begin

      { translate color value }
      TempInt := LookUpColor(StripChars(S, 'TEXTCOLOR'));

      { if valid color }
      If TempInt > 0 then
        ColorOfText := TempInt or 8  { make sure high-intensity }

    End { Else If }


    { check for multi-color stuff }
    Else If Pos('SOLID', S) = 1 then
      TranslateSolid(S)


    { check for timing data output file }
    Else If CheckAndSetString(S, 'TIMEFILE', Parm, TimeFName) then

    { check for delay value }
    Else If Pos('DELAY', S) = 1 then begin

      { if only one parameter, get delay value }
      If GetParms(S, 'DELAY') = 1 then
        If (Parms[1] > 0) and (Parms[1] < 2147480.0) then begin
          DelayGiven := True;
          DelayMilli := Round(Parms[1] * 1000)
        End { If }

    End { Else If }


    { check for lighting modifier }
    Else If CheckAndSetFloat(S, 'INTENSITY', Intensity) then


    { check for scaling factors }
    Else If CheckAndSetFloat(S, 'XSCALE', XScale) then
      XScale := Abs(XScale)
    Else If CheckAndSetFloat(S, 'YSCALE', YScale) then
      YScale := Abs(YScale)
    Else If CheckAndSetFloat(S, 'ZSCALE', ZScale) then
      ZScale := Abs(ZScale)

    Else If CheckAndSetFloat(S, 'HSCALE', HScale) then
      HScale := Abs(HScale)
    Else If CheckAndSetFloat(S, 'VSCALE', VScale) then
      VScale := Abs(VScale)

    Else If CheckAndSetFloat(S, 'SCALE', UserScale) then
      UserScale := Abs(UserScale)


    { check for lighting parameter enumeration }
    Else If Pos('LIGHT[', S) = 1 then
      ParseLighting(StripChars(S, 'LIGHT['))


    Else If Pos('SPOT', S) = 1 then
      ParseSpotLighting(StripChars(S, 'SPOTLIGHT'))


    { check for grid corner values }
    Else If Pos('CORNER', S) = 1 then begin

      { set viewpoint if there were the right number of parameters }
      If GetParms(S, 'CORNERS') = 4 then begin
        XMin := Parms[1];
        YMin := Parms[2];
        XMax := Parms[3];
        YMax := Parms[4];
        XMinStr := 'XMin = ' + ParmText[1];
        YMinStr := 'YMin = ' + ParmText[2];
        XMaxStr := 'XMax = ' + ParmText[3];
        YMaxStr := 'YMax = ' + ParmText[4]
      End { If }

    End { Else }


    { go for grid boundary adjustments }
    Else If CheckAndSetFloat(S, 'XMIN', XMin) then
      XMinStr := 'XMin = ' + ParmText[1]
    Else If CheckAndSetFloat(S, 'YMIN', YMin) then
      YMinStr := 'YMin = ' + ParmText[1]
    Else If CheckAndSetFloat(S, 'XMAX', XMax) then
      XMaxStr := 'XMax = ' + ParmText[1]
    Else If CheckAndSetFloat(S, 'YMAX', YMax) then
      YMaxStr := 'YMax = ' + ParmText[1]


    { check for evaluation clipping range }
    Else If CheckAndSetFloat(S, 'MINHEIGHT', MinHeight) then
    Else If CheckAndSetFloat(S, 'MAXHEIGHT', MaxHeight) then
    Else If Pos('ZCLIP', S) = 1 then begin

      { set viewpoint if there were the right number of parameters }
      If GetParms(S, 'ZCLIPPING') = 2 then begin
        MinHeight := Parms[1];
        MaxHeight := Parms[2];
      End { If }

    End { Else }


    { check for symmetric range }
    Else If Pos('RANGE', S) = 1 then begin

      { if it worked, set the range }
      If GetParms(S, 'RANGE') = 1 then begin
        TempFloat := Abs(TempFloat);
        XMin := -Parms[1];
        YMin := -Parms[1];
        XMax := Parms[1];
        YMax := Parms[1];
        XMinStr := 'XMin = -(' + ParmText[1] + ')';
        YMinStr := 'YMin = -(' + ParmText[1] + ')';
        XMaxStr := 'XMax = ' + ParmText[1];
        YMaxStr := 'YMax = ' + ParmText[1]
      End { If }

    End { Else If }

    { check for x-axis symmetric range }
    Else If Pos('XRANGE', S) = 1 then begin

      I := GetParms(S, 'XRANGE');

      { if it worked, set the range }
      If I = 1 then begin
        XMin := -Parms[1];
        XMax := Parms[1];
        XMinStr := 'XMin = -(' + ParmText[1] + ')';
        XMaxStr := 'XMax = ' + ParmText[1];
      End { If }
      Else If I = 2 then begin
        XMin := Parms[1];
        XMax := Parms[2];
        XMinStr := 'XMin = ' + ParmText[1];
        XMaxStr := 'XMax = ' + ParmText[2];
      End; { Else }

    End { Else If }

    { check for y-axis symmetric range }
    Else If Pos('YRANGE', S) = 1 then begin

      I := GetParms(S, 'YRANGE');

      { if it worked, set the range }
      If I = 1 then begin
        YMin := -Parms[1];
        YMax := Parms[1];
        YMinStr := 'YMin = -(' + ParmText[1] + ')';
        YMaxStr := 'YMax = ' + ParmText[1];
      End { If }
      Else If I = 2 then begin
        YMin := Parms[1];
        YMax := Parms[2];
        YMinStr := 'YMin = ' + ParmText[1];
        YMaxStr := 'YMax = ' + ParmText[2];
      End; { Else }

    End { Else If }

    { set grid range based on minimum point and size }
    Else If Pos('MINSIZE', S) = 1 then begin

      { if the right number were specified, set grid }
      If GetParms(S, 'MINSIZE') = 3 then begin
        XMin := Parms[1];
        YMin := Parms[2];
        XMax := Parms[1] + Parms[3];
        YMax := Parms[2] + Parms[3];
        XMinStr := 'XMin = ' + ParmText[1];
        YMinStr := 'YMin = ' + ParmText[2];
        XMaxStr := 'XMax = (' + ParmText[1] + ')+(' + ParmText[3] + ')';
        YMaxStr := 'YMax = (' + ParmText[2] + ')+(' + ParmText[3] + ')';
      End { If }

    End { Else }


    { set grid range based on maximum point and size }
    Else If Pos('MAXSIZE', S) = 1 then begin

      { if the right number were specified, set grid }
      If GetParms(S, 'MAXSIZE') = 3 then begin
        XMin := Parms[1] - Parms[3];
        YMin := Parms[2] - Parms[3];
        XMax := Parms[1];
        YMax := Parms[2];
        XMinStr := 'XMin = (' + ParmText[1] + ')-(' + ParmText[3] + ')';
        YMinStr := 'YMin = (' + ParmText[2] + ')-(' + ParmText[3] + ')';
        XMaxStr := 'XMax = ' + ParmText[1];
        YMaxStr := 'YMax = ' + ParmText[2];
      End { If }

    End { Else }


    Else If Pos('DISA', S) = 1 then
      DoDisa := True

    Else If CheckAndSetFlag(S, 'VIEWIGNORE', IgnoreViewPointRange) then

    Else If CheckAndSetFlag(S, 'PRECOMPUTE', PreCompute) then

    Else If CheckAndSetFlag(S, 'BLUR', Blur) then

    { check for animation parameters }
    Else If Pos('ANIMATE', S) = 1 then begin

      If GetParms(S, 'ANIMATE') = 3 then begin

        { if the number of frames is in a reasonable range }
        If (Parms[1] > 0) and (Parms[1] < 32700.0) then begin

          { set the animation parameters }
          DoAnimate  := True;
          AnimFrames := Round(Parms[1]);
          AnimStart  := Parms[2];
          AnimStop   := Parms[3];

        End; { If }

      End; { If }

    End { Else If }

    { check for delay time between frames }
    Else If Pos('ANIMDELAY', S) = 1 then begin

      If GetParms(S, 'ANIMDELAY') = 0 then ;

      { set delay if only one parm in range 0..65535 }
      If (NumParms = 1) and (Parms[1] <= 65535) and (Parms[1] >= 0) then
        AnimDelay := Trunc(Parms[1]);

    End { Else If }

    { check for frame to replace }
    Else If Pos('ANIMSETFRAME', S) = 1 then begin

      { strip parm from front end (in multiple alternate forms) }
      If GetParms(S, 'ANIMSETFRAME') = 0 then ;

      { set frame num if only one parm in range 0..32766 }
      If (NumParms = 1) and (Parms[1] <= 32766) and (Parms[1] >= 0) then
        AnimFrame := Trunc(Parms[1]);

    End { Else If }

    { check for view file name for save }
    Else If Pos('ANIMSAVE', S) = 1 then begin
      AnimName := Copy(StripChars(S, 'ANIMSAVE'), 1, 120);
      AnimSave := AnimName <>'';
    End { Else If }

    { set starting animation parameter }
    Else If CheckAndSetFloat(S, 'ANIMSTART', AnimStart) then
      DoAnimate := True

    { set ending animation parameter }
    Else If CheckAndSetFloat(S,'ANIMSTOP', AnimStop) then
      DoAnimate := True

    { sets the parameter for a single animtion frame recompute }
    Else If CheckAndSetFloat(S, 'ANIMTIME', AnimParm) then

    { check for number of frames in the animation }
    Else If Pos('ANIMFRAMES', S) = 1 then begin

      If GetParms(S, 'ANIMFRAMES') = 0 then ;

      { set delay if only one parm in range 0..65535 }
      If (NumParms = 1) and (Parms[1] <= 65535) and (Parms[1] >= 0) then begin
        AnimFrames:= Trunc(Parms[1]);
        DoAnimate := True;
      End; { If }

    End { Else If }

    { check for animation playback }
    Else If Pos('ANIMSHOW', S) = 1 then begin

      If GetSParms(S, 'ANIMSHOW') = 1 then begin

        AnimName := ParmText[1];
        AnimLoad := AnimName <>'';

      End; { If }

    End { Else If }

    { check for direction reversal during playback }
    Else If CheckAndSetFlag(S, 'ANIMREVERSE', AnimReverse) then

    { check for the number of calcualtion frames }
    Else If Pos('CALCFRAME', S) = 1 then begin
      If GetParms(S, 'CALCFRAMES') = 1 then begin
        If (Parms[1] >= 1.0) and (Parms[1] <= 32767) then
          MaxCalcFrames := Trunc(Parms[1])
        Else
          MaxCalcFrames := 1;
      End; { If }
    End { Else If }

    { check for viewpoint syntax A }
    Else If Pos('VIEW', S) = 1 then begin

      { set focus point if there were the right number of parameters }
      If GetParms(S, 'VIEWPOINT') = 3 then begin
        ViewPoint.X := Parms[1];
        ViewPoint.Y := Parms[2];
        ViewPoint.Z := Parms[3];
        ViewText[1] := ParmText[1];
        ViewText[2] := ParmText[2];
        ViewText[3] := ParmText[3];
        ViewPolar   := False;
        ViewParse   := False;
      End { If }
      Else If GetSParms(S, 'VIEWPOINT') = 3 then begin
        ViewText[1] := ParmText[1];
        ViewText[2] := ParmText[2];
        ViewText[3] := ParmText[3];
        ViewPolar   := False;
        ViewParse   := True;
      End; { Else }

    End { Else If }


    { check for syntax B viewpoint entry }
    Else If Pos('VPOINT', S) = 1 then begin

      { set viewpoint if there were the right number of parameters }
      If GetParms(S, 'VPOINT') = 3 then begin
        ViewPoint.X := Parms[1];
        ViewPoint.Y := Parms[2];
        ViewPoint.Z := Parms[3];
        ViewText[1] := ParmText[1];
        ViewText[2] := ParmText[2];
        ViewText[3] := ParmText[3];
        ViewPolar   := False;
        ViewParse   := False;
      End { If }
      Else If GetSParms(S, 'VPOINT') = 3 then begin
        ViewText[1] := ParmText[1];
        ViewText[2] := ParmText[2];
        ViewText[3] := ParmText[3];
        ViewPolar   := False;
        ViewParse   := True;
      End; { Else }

    End { Else If }


    { check for viewpoint syntax C }
    Else If Pos('AZELR', S) = 1 then begin

      { if the right number were specified, then set viewpoint,
        converting from polar to rectangular coordinates }
      If GetParms(S, 'AZELR') = 3 then begin
        PolarToRect(Parms[1], Parms[2], Parms[3]);
        ViewPoint.X := Parms[1];
        ViewPoint.Y := Parms[2];
        ViewPoint.Z := Parms[3];
        ViewPolar   := True;
        ViewParse   := False;
      End { If }
      Else If GetSParms(S, 'AZELR') = 3 then begin
        ViewText[1] := ParmText[1];
        ViewText[2] := ParmText[2];
        ViewText[3] := ParmText[3];
        ViewPolar   := True;
        ViewParse   := True;
      End; { Else }

    End { Else If }


    { check for complete focus point entry }
    Else If Pos('FOCUS', S) = 1 then begin

      { set focus point if there were the right number of parameters }
      If GetParms(S, 'FOCUSPOINT') = 3 then begin
        FocusSet     := True;
        Focus.X      := Parms[1];
        Focus.Y      := Parms[2];
        Focus.Z      := Parms[3];
        FocusText[1] := ParmText[1];
        FocusText[2] := ParmText[1];
        FocusText[3] := ParmText[1];
        FocusParse   := False;
      End { If }
      Else If GetSParms(S, 'FOCUSPOINT') = 3 then begin
        FocusSet     := True;
        FocusText[1] := ParmText[1];
        FocusText[2] := ParmText[2];
        FocusText[3] := ParmText[3];
        FocusParse   := True;
      End { Else If }

    End { Else If }


    { check for complete camera entry }
    Else If Pos('CAMERA', S) = 1 then begin

      { set focus point if there were the right number of parameters }
      If GetParms(S, 'CAMERA') = 6 then begin

        FocusSet := True;

        ViewPoint.X  := Parms[1];
        ViewPoint.Y  := Parms[2];
        ViewPoint.Z  := Parms[3];
        ViewText[1]  := ParmText[1];
        ViewText[2]  := ParmText[2];
        ViewText[3]  := ParmText[3];
        ViewPolar    := False;
        ViewParse    := False;

        Focus.X      := Parms[4];
        Focus.Y      := Parms[5];
        Focus.Z      := Parms[6];
        FocusText[1] := ParmText[4];
        FocusText[2] := ParmText[5];
        FocusText[3] := ParmText[6];
        FocusPolar   := False;
        FocusParse   := False;

      End { If }
      Else If GetSParms(S, 'CAMERA') = 6 then begin

        FocusSet := True;

        ViewText[1]  := ParmText[1];
        ViewText[2]  := ParmText[2];
        ViewText[3]  := ParmText[3];
        ViewPolar    := False;
        ViewParse    := True;

        FocusText[1] := ParmText[4];
        FocusText[2] := ParmText[5];
        FocusText[3] := ParmText[6];
        FocusPolar   := False;
        FocusParse   := True;

      End; { If }

    End { Else If }

    { check for load or save command }
    Else If CheckFileOper(S) then

    { see if a command ile was placed bare on cmd line }
    Else If CheckBareCmdFile(S) then

    { else assume that any parameter not processed must be an expression }
    Else begin

      { check the input for being an expression }
      Parse(S, Error, Posn);

      { if the system recognized the expression and no assignment }
      If Error = NoAssignExpr then begin

        { remember what the string looks like }
        FuncStr := Parm;

        { and no error }
        Error := NoError;

      End { If }

      { if the system couldn't recognize the expression, unknown command }
      Else If Error <> NoError then begin
        Writeln;
        Writeln(Parm);
        Writeln('^':Posn, '-- ', EvalErrMsg(Error));
        Abort('Invalid function')
      End { If }

    End { Else }

    (* COMMAND CHECKING ENDS HERE *)

  End; { Procedure CheckParm }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

Begin

  { set some not-easily expressible constants }
  CheckParm(DefaultViewPoint);
  CheckParm(DefaultFocus);
  CheckParm(XMinStr);
  CheckParm(YMinStr);
  CheckParm(XMaxStr);
  CheckParm(YMaxStr);

  { go through each parameter, setting given data }
  For I := 1 to ParamCount do
    CheckParm(ParamStr(I));

End; { Procedure GetCommandLine }


End. { Unit GetCmd }
