Unit GrText;

(****************************************************************************
**                                                                         **
**  Draw stroked font in the coordinate system of the function display     **
**                                                                         **
****************************************************************************)

Interface

Uses
  Vectors;

Procedure DrawText(BaseX,BaseY,BaseZ, XRot,YRot,ZRot : Float; S : String);
Function TextLength(S : String) : Float;

Implementation

Uses
  View,
  FDecls,
  Graph,
  Graphics;

Const
  PrefixSize = 128;

Type
  FontHeader = Record
    Signature     : Char;     { signature byte }
    NumChars      : Integer;  { number of characters in file }
    unused1       : Char;     { unused }
    First         : Char;     { first character in file }
    CDefs         : Integer;  { offset to character definitions }
    ScanFlag      : Boolean;  { set is scanable }
    OriginToCap,              { height from origin to top of capitol }
    OriginToBase,             { height from origin to baseline }
    OriginToDesc  : ShortInt; { height from origin to bottom of descender }
    FontName      : Array[1..4] of Char; { short font name }
    unused2       : Char;     { unused }
  End; { Record }

  FOfsArray =  Array[0..255] of Word;
  FWdtArray =  Array[0..255] of Byte;

Var
  FHdr  : ^FontHeader; { pointer to font header }
  FOfs  : ^FOfsArray;  { pointer to character offset array }
  FWdt  : ^FWdtArray;  { pointer to width table (byte-wide) }
  FDat  : Pointer;     { pointer to stroke data }
  Last  : Char;        { last defined character }


{ attach font data }
Procedure FontData;
External; {$L FONT.OBJ}

Var
  FontScale : Float; { "unit" scaling factor for text }

{-------------------------------------------------------------------------}

Procedure DrawText;
{
  Draw text on the screen in the position and with the rotation desired.
}
Const
  CharEnd = $0000;
  Move    = $0080;
  DoScan  = $8000;
  Draw    = $8080;
  CmdMask = $8080;

Var
  P           : ^Word;    { current position in definition }
  Code        : Word;     { current data point }
  IX,IY,                  { integer coordinates for text }
  I           : Integer;  { index into string }

  OldViewMat,             { current viewing matrix }
  t1,t2,                  { scratch matrices }
  RotMat      : Matrix;   { current rotation matrix }

  FF,                     { FontSize * FontScale }
  CW,                     { character width * FF }
  CosX,SinX,
  CosY,SinY,
  CosZ,SinZ,
  X1,Y1,Z1,               { current raw XYZ point in font }
  X,Y,Z       : Float;    { current transformed XYZ point in font }
  SX,SY,                  { current screen XY point }
  OX,OY       : Single;   { previous XY point }
  Ch          : Char;     { current character }

Begin

  { precompute text scaling factor }
  FF := FontScale * FontSize;

  { precompute rotation factors }
  CosX := cos(XRot);
  SinX := sin(XRot);
  CosY := cos(YRot);
  SinY := sin(YRot);
  CosZ := cos(ZRot);
  SinZ := sin(ZRot);

  { use matrix operations to get correct things }

  { rotate about X }
  RotMat      := IdentMatrix;
  RotMat[2,2] :=  CosX;
  RotMat[3,3] :=  CosX;
  RotMat[2,3] :=  SinX;
  RotMat[3,2] := -SinX;

  { rotate about Y }
  t1      := IdentMatrix;
  t1[1,1] :=  CosY;
  t1[3,3] :=  CosY;
  t1[1,3] :=  SinY;
  t1[3,1] := -SinY;
  MultMat(t2,RotMat,t1);
  RotMat := t2;

  { rotate about Z }
  t1      := IdentMatrix;
  t1[1,1] :=  CosZ;
  t1[2,2] :=  CosZ;
  t1[2,1] :=  SinZ;
  t1[1,2] := -SinZ;
  MultMat(t2,RotMat,t1);
  RotMat := t2;

  { set correct color for text }
  SetColor(ColorOfText);

  { process each cahracter in string }
  For I := 1 to Length(S) do begin

    { get current character }
    Ch := S[I];

    { if the current character is defined }
    If (Ch >= FHdr^.First) and (Ch <= Last) then begin

      { locate the start of the data }
      P := Pointer(LongInt(FDat) + FOfs^[Ord(Ch) - Ord(FHdr^.First)]);

      Repeat

        { get word to work with }
        Code := P^;

        { move to next word }
        Inc(LongInt(P),SizeOf(Word));

        { extract, scale, and translate X }
        IX := Code and 127;
        If (IX and $40 <> 0) then
          IX := IX or $FF80;
        X1 := IX * FF;

        { extract, scale and translate Y }
        IY := (Code shr 8) and 127;
        If (IY and $40 <> 0) then
          IY := IY or $FF80;
        Y1 := IY * FF;

        Z1 := 0;

        { rotate to correct position }
        X := BaseX + RotMat[1,1]*X1 + RotMat[2,1]*Y1 + RotMat[3,1]*Z1;
        Y := BaseY + RotMat[1,2]*X1 + RotMat[2,2]*Y1 + RotMat[3,2]*Z1;
        Z := BaseZ + RotMat[1,3]*X1 + RotMat[2,3]*Y1 + RotMat[3,3]*Z1;

        { convert character XYZ to screen XY }
        If Transform(X,Y,Z, SX,SY) then begin

          { perform coded draw command if required }
          If CmdMask and Code = Draw then
            DrawLine(OX,OY, SX,SY);

        End; { If }

        { move position }
        OX := SX;
        OY := SY

      { go until all commands processed }
      Until CmdMask and Code = CharEnd;

      { figure the width of the current character }
      CW := FF * FWdt^[Ord(Ch) - Ord(FHdr^.First)];

      { move base point over to start of next character }
      BaseX := BaseX + RotMat[1,1]*CW;
      BaseY := BaseY + RotMat[1,2]*CW;
      BaseZ := BaseZ + RotMat[1,3]*CW;

    End { If }

  End { For }

End; { Procedure DrawText }

{-------------------------------------------------------------------------}

Function TextLength;
{
  Calculate the length of a text string in the stroked font
}
Var
  I : Integer;
  L : Float;
Begin

  L := 0.0;

  For I := 1 to Length(S) do

    If (S[I] >= FHdr^.First) and (S[I] <= Last) then

      L := L + FWdt^[Ord(S[I]) - Ord(FHdr^.First)];

  TextLength := L * FontScale * FontSize

End; { Function TextLength }

{-------------------------------------------------------------------------}

Begin

  { init font pointers }
  FHdr := Pointer(LongInt(@FontData) + PrefixSize);
  FOfs := Pointer(LongInt(FHdr) + SizeOf(FontHeader));
  FWdt := Pointer(LongInt(FOfs) + FHdr^.NumChars * 2);
  FDat := Pointer(LongInt(FHdr) + FHdr^.CDefs);

  { manufacture a scaling factor }
  FontScale := 1.0 / (FHdr^.OriginToCap - FHdr^.OrigintoDesc);

  { determine the last defined character }
  Last := Chr(Ord(FHdr^.First) + FHdr^.NumChars)

End.
