Unit ParsEval;

(****************************************************************************
**                                                                         **
**  Parsing and execution of expressions.                                  **
**                                                                         **
****************************************************************************)

{

    The virtual machine in the evaluate routine is the typical stack-based
  critter.  Rather than TOS being at the top of the stack, however, it is
  kept in a variable called TOS, which helps speed things up a trifle by
  removing may array indexing operations.  If it was possible to keep TOS
  in an 8087 register, it would go even faster, but that's not possible
  with this compiler.
    The code it uses consists of single-byte opcodes, portions of which are
  used as indices into tables for loading values.  For executing operations,
  the system simply keeps comparing opcodes with constants until it find a
  match.  Not all of the defined operations in the 0-63 range are used by
  the execution machinery; some are simply token values returned by the
  scanner.

    opcode allocation:

      000-063   operators, comparisons, jumps, return, etc.

      064-128   load constant

      129-191   system functions

      192-255   load expression value (exec expression if required)


    All of the variables carry a tag indicating if the data is valid or not.
  If the value of a variable is invalid, it performs whatever action is
  required to make it valid, including executing a subroutine.  Once the
  data is valid, it is pushed onto the stack.

    There is still a problem with constant folding.  Ideally, it would be
  a simple matter to determine if an entire expression is a constant (for
  whole-expression checking, it's just a matter of searching for variables
  in the code), but because of the way constant table space is reused, it is
  somewhat difficult to do this.  Perhaps a simple left-to-right routine
  that checks precedence would also work.  If it works on an expression like
  sin(((1 + 2) * (3 + 4)) ^ (5 - 6), it should work for any).
    Constant expressions should be evaluated by the parser, not by the
  Evaluate routine.  This would, however, necessitate some way to evaluate the
  system functions, like sin and asinh.  Perhaps moving the guts of the main
  CASE statement (at least with regards to the math functions) to a separate
  routine, which takes two source and one destination paramters, would be
  the best.  Unfortunately, this wouldn't be the best way to do the "normal"
  operations (while the main program is executing).

    The current evaluation mechanism is being slowed down by the Turbo Pascal
  Case evaluation mechanism (testing sequentially).  A lookup-table of
  functions might speed thigns up greatly, or at least make the evaluation
  cost of each operation the same.
}

Interface

Uses
  Vectors;

Const

  ConstError    = -3.2718281828459045-37;  { error in constant expression }

  { error codes }
  NoError          =  0;
  NeedMoreChars    =  1;
  BadChar          =  2;
  BadNumber        =  3;
  InvalidToken     =  4;
  InvalidAssign    =  5;
  UnknownID        =  6;
  OperatorExpected =  7;
  OperandExpected  =  8;
  AssignExpected   =  9;
  ConstExpected    = 10;
  CommaExpected    = 11;
  TooManyConsts    = 12;
  TooManyExprs     = 13;
  CodeOverflow     = 14;
  RParenExpected   = 15;
  LParenExpected   = 16;
  TooManyRParens   = 17;
  MainRedefinition = 18;
  IdentExpected    = 19;
  LabelRedefined   = 20;
  NotInConst       = 21;
  SeriesNotDeepEnough = 22;
  LastError        = 23;
  NoAssignExpr     = 16000;  { not an error, but gets passed around like one }

Procedure Parse(InStr : String; Var Error,Posn : Integer);
Function  EvalConst(Expr : String) : Float;
Function  Evaluate(X, Y : Float) : Float;
Function  IEvaluate(X, Y : Float; IX, IY : Integer) : Float;
Function  EvalErrMsg(Error : Integer) : String;
Function  MainFunctionDefined : Boolean;
Procedure Disassemble;

Function GetVarValue(VarName : String) : Float;


Implementation

Uses
  CRT,       { for access to DirectVideo }
  FDecls,    { for misc data }
  MathUnit,  { math functions }
  QuadFrac;  { fractal stuff }

Const
  LoadBase    = 192;

{ symbols from scanner and for virtual machine }
Type
  TokenType = (

    NOP,

    { arithmetic operators }
    AOADD,
    AOSUB,
    AODIV,
    AOMUL,
    AOEXP,
    AONEG,

    { logical operators }
    LOEQ,   { =    == .eq.   }
    LONE,   { <>   != .ne    }
    LOLT,   { <       .lt.   }
    LOGT,   { >       .gt.   }
    LOLE,   { <=      .le.   }
    LOGE,   { >=      .ge.   }
    LCAND,  { &    && .and.  }
    LCOR,   { |    || .or.   }
    LCXOR,  {         .xor.  }
    LCNOT,  { !       .not.  }

    { function tokens }
    FSIN,
    FCOS,
    FTAN,
    FASIN,
    FACOS,
    FATAN,
    FSINH,
    FCOSH,
    FTANH,
    FASINH,
    FACOSH,
    FATANH,
    FEXP,
    FEXP10,
    FLN,
    FLOG,
    FSQRT,
    FCBRT,
    FSQR,
    FABS,
    FROUND,
    FFLOOR,
    FCEIL,
    FFACT,
    FFRAC,
    FATAN2,
    FSINC,
    FBESSEL,
    FGAMMA,
    FODD,
    FSIGN,
    FDIST,
    FANGLE,
    FSEC,
    FCSC,
    FCOT,
    FASEC,
    FACSC,
    FACOT,
    FSECH,
    FCSCH,
    FCOTH,
    FASECH,
    FACSCH,
    FACOTH,
    FQUADFRAC,
    FDECIBEL,
    FSURFACE,
    FISURFACE,

    FSIND,
    FCOSD,
    FTAND,
    FASIND,
    FACOSD,
    FATAND,
    FSECD,
    FCSCD,
    FCOTD,
    FASECD,
    FACSCD,
    FACOTD,

    { zero-parameter functions }
    FVPI,
    FVE,
    FVX,
    FVY,
    FVIX,
    FVIY,
    FVXMIN,
    FVXMAX,
    FVYMIN,
    FVYMAX,
    FVIXMAX,
    FVIYMAX,
    FVR,
    FVT,
    FVMANDEL,
    FVJULIA,
    FVRND,
    FVCFRAME,
    FVCMAXFRAME,

    FVVIEWX,     { viewpoint location }
    FVVIEWY,
    FVVIEWZ,
    FVVIEWAZ,    { azimuth angle (radians) to viewpoint from 0,0,0 }
    FVVIEWEL,    { elevation angle (radians) to viewpoint from 0,0,0 }
    FVVIEWR,     { range to viewpoint from 0,0,0 }
    FVFOCUSX,    { location of focus point }
    FVFOCUSY,
    FVFOCUSZ,
    FVFOCUSAZ,   { azimuth angle (radians) to focus from 0,0,0 }
    FVFOCUSEL,   { elevation angle (radians) to focus from 0,0,0 }
    FVFOCUSR,    { range to focus from 0,0,0 }
    FVANIMTIME,  { current animation time }
    FVANIMSTART, { animation start time }
    FVANIMSTOP,  { animation stop time }

    { "other" functions (series) }
    FVSSUM,
    FVSPROD,

    { scanner-only tokens }
    NUMBER,
    USERVAR,
    IDENT,
    LPAREN,
    RPAREN,
    ENDOF,
    COMMA,
    LIF,       { ?    .if.   }
    LELSE,     { :    .else. }
    LGOTO,     {      .goto. }
    LLABEL,    { ::text      }
    LOADCONST, { load number from code space }

    LOADVAR,   { load variable }
    ASSIGN,    { assignment token (:=) or store to variable }

    { virtual machine only instructions }
    VMRET,     { return from routine }
    VMJMP,     { set PC to address }

    VMJZ,      { jump to address if TOS = 0 - pops TOS }
    VMJLZ,     { jump to address if TOS < 0 - pops TOS }
    VMJGZ,     { jump to address if TOS > 0 - pops TOS }

    VMJEQ,     { jump to address if TOS-1 = TOS  - pops TOS and TOS - 1 }
    VMJLT,     { jump to address if TOS-1 < TOS  - pops TOS and TOS - 1 }
    VMJGT,     { jump to address if TOS-1 > TOS  - pops TOS and TOS - 1 }
    VMJLE,     { jump to address if TOS-1 <= TOS - pops TOS and TOS - 1 }
    VMJGE,     { jump to address if TOS-1 >= TOS - pops TOS and TOS - 1 }

    ADDSTART,  { sets up summation series }
    ADDCONT,   { continues summation series }
    MULSTART,  { sets up product series }
    MULCONT,   { continues product series }

    LSTART,    { load series index start }
    LSTEP,     { load series index step value }
    LFINISH,   { load series index finish }
    LINDEX,    { load series index value }
    LRESULT    { load series index start }

  );

Const
  CodeSize    = 1023;          { size of code }
  CodeTop     : Integer = 0;   { top of code space (start of unused) }
  MainEntry   : Integer = -1;  { no entry point for main expression just yet }

Var
  Code        : Array[0..CodeSize] of Byte; { code space }

Const
  NumSysSym = 96;
  SysSym : Array[0..NumSysSym - 1] of Record { system functions }
    ID     : String[8];  { identifying text }
    Token  : TokenType;  { token for this identifier }
    NParms : ShortInt    { # parameters }
  End = (
    ( ID : 'SIN'       ; Token : FSIN        ; NParms : 1 ),
    ( ID : 'COS'       ; Token : FCOS        ; NParms : 1 ),
    ( ID : 'TAN'       ; Token : FTAN        ; NParms : 1 ),
    ( ID : 'ASIN'      ; Token : FASIN       ; NParms : 1 ),
    ( ID : 'ACOS'      ; Token : FACOS       ; NParms : 1 ),
    ( ID : 'ATAN'      ; Token : FATAN       ; NParms : 1 ),
    ( ID : 'SINH'      ; Token : FSINH       ; NParms : 1 ),
    ( ID : 'COSH'      ; Token : FCOSH       ; NParms : 1 ),
    ( ID : 'TANH'      ; Token : FTANH       ; NParms : 1 ),
    ( ID : 'ASINH'     ; Token : FASINH      ; NParms : 1 ),
    ( ID : 'ACOSH'     ; Token : FACOSH      ; NParms : 1 ),
    ( ID : 'ATANH'     ; Token : FATANH      ; NParms : 1 ),
    ( ID : 'EXP'       ; Token : FEXP        ; NParms : 1 ),
    ( ID : 'EXP10'     ; Token : FEXP10      ; NParms : 1 ),
    ( ID : 'LN'        ; Token : FLN         ; NParms : 1 ),
    ( ID : 'LOG'       ; Token : FLOG        ; NParms : 1 ),
    ( ID : 'SQRT'      ; Token : FSQRT       ; NParms : 1 ),
    ( ID : 'CBRT'      ; Token : FCBRT       ; NParms : 1 ),
    ( ID : 'SQR'       ; Token : FSQR        ; NParms : 1 ),
    ( ID : 'ABS'       ; Token : FABS        ; NParms : 1 ),
    ( ID : 'ROUND'     ; Token : FROUND      ; NParms : 1 ),
    ( ID : 'FLOOR'     ; Token : FFLOOR      ; NParms : 1 ),
    ( ID : 'CEIL'      ; Token : FCEIL       ; NParms : 1 ),
    ( ID : 'FACT'      ; Token : FFACT       ; NParms : 1 ),
    ( ID : 'FRAC'      ; Token : FFRAC       ; NParms : 1 ),
    ( ID : 'ATAN2'     ; Token : FATAN2      ; NParms : 2 ),
    ( ID : 'SINC'      ; Token : FSINC       ; NParms : 1 ),
    ( ID : 'BESSEL'    ; Token : FBESSEL     ; NParms : 2 ),
    ( ID : 'GAMMA'     ; Token : FGAMMA      ; NParms : 1 ),
    ( ID : 'ODD'       ; Token : FODD        ; NParms : 1 ),
    ( ID : 'SIGN'      ; Token : FSIGN       ; NParms : 1 ),
    ( ID : 'DIST'      ; Token : FDIST       ; NParms : 2 ),
    ( ID : 'ANGLE'     ; Token : FANGLE      ; NParms : 2 ),
    ( ID : 'SEC'       ; Token : FSEC        ; NParms : 1 ),
    ( ID : 'CSC'       ; Token : FCSC        ; NParms : 1 ),
    ( ID : 'COT'       ; Token : FCOT        ; NParms : 1 ),
    ( ID : 'ASEC'      ; Token : FSEC        ; NParms : 1 ),
    ( ID : 'ACSC'      ; Token : FCSC        ; NParms : 1 ),
    ( ID : 'ACOT'      ; Token : FACOT       ; NParms : 1 ),
    ( ID : 'SECH'      ; Token : FSECH       ; NParms : 1 ),
    ( ID : 'CSCH'      ; Token : FCSCH       ; NParms : 1 ),
    ( ID : 'COTH'      ; Token : FCOTH       ; NParms : 1 ),
    ( ID : 'ASECH'     ; Token : FASECH      ; NParms : 1 ),
    ( ID : 'ACSCH'     ; Token : FACSCH      ; NParms : 1 ),
    ( ID : 'ACOTH'     ; Token : FACOTH      ; NParms : 1 ),
    ( ID : 'QUADFRAC'  ; Token : FQUADFRAC   ; NParms : 4 ),
    ( ID : 'MANDEL'    ; Token : FVMANDEL    ; NParms : 0 ),
    ( ID : 'JULIA'     ; Token : FVJULIA     ; NParms : 0 ),
    ( ID : 'X'         ; Token : FVX         ; NParms : 0 ),
    ( ID : 'Y'         ; Token : FVY         ; NParms : 0 ),
    ( ID : 'R'         ; Token : FVR         ; NParms : 0 ),
    ( ID : 'T'         ; Token : FVT         ; NParms : 0 ),
    ( ID : 'XMIN'      ; Token : FVXMIN      ; NParms : 0 ),
    ( ID : 'YMIN'      ; Token : FVYMIN      ; NParms : 0 ),
    ( ID : 'XMAX'      ; Token : FVXMAX      ; NParms : 0 ),
    ( ID : 'YMAX'      ; Token : FVYMAX      ; NParms : 0 ),
    ( ID : 'RND'       ; Token : FVRND       ; NParms : 0 ),
    ( ID : 'PI'        ; Token : FVPI        ; NParms : 0 ),
    ( ID : 'SSUM'      ; Token : FVSSUM      ; NParms : 0 ),
    ( ID : 'SPROD'     ; Token : FVSPROD     ; NParms : 0 ),
    ( ID : 'VIEWX'     ; Token : FVVIEWX     ; NParms : 0 ),
    ( ID : 'VIEWY'     ; Token : FVVIEWY     ; NParms : 0 ),
    ( ID : 'VIEWZ'     ; Token : FVVIEWZ     ; NParms : 0 ),
    ( ID : 'VIEWAZ'    ; Token : FVVIEWAZ    ; NParms : 0 ),
    ( ID : 'VIEWEL'    ; Token : FVVIEWEL    ; NParms : 0 ),
    ( ID : 'VIEWR'     ; Token : FVVIEWR     ; NParms : 0 ),
    ( ID : 'FOCUSX'    ; Token : FVFOCUSX    ; NParms : 0 ),
    ( ID : 'FOCUSY'    ; Token : FVFOCUSY    ; NParms : 0 ),
    ( ID : 'FOCUSZ'    ; Token : FVFOCUSZ    ; NParms : 0 ),
    ( ID : 'FOCUSAZ'   ; Token : FVFOCUSAZ   ; NParms : 0 ),
    ( ID : 'FOCUSEL'   ; Token : FVFOCUSEL   ; NParms : 0 ),
    ( ID : 'FOCUSR'    ; Token : FVFOCUSR    ; NParms : 0 ),
    ( ID : 'ANIMTIME'  ; Token : FVANIMTIME  ; NParms : 0 ),
    ( ID : 'ANIMSTART' ; Token : FVANIMSTART ; NParms : 0 ),
    ( ID : 'ANIMSTOP'  ; Token : FVANIMSTOP  ; NParms : 0 ),
    ( ID : 'DECIBEL'   ; Token : FDECIBEL    ; NParms : 1 ),
    ( ID : 'SURFACE'   ; Token : FSURFACE    ; NParms : 2 ),
    ( ID : 'ISURFACE'  ; Token : FISURFACE   ; NParms : 2 ),
    ( ID : 'IX'        ; Token : FVIX        ; NParms : 0 ),
    ( ID : 'IY'        ; Token : FVIY        ; NParms : 0 ),
    ( ID : 'IXMAX'     ; Token : FVIXMAX     ; NParms : 0 ),
    ( ID : 'IYMAX'     ; Token : FVIYMAX     ; NParms : 0 ),
    ( ID : 'SIND'      ; Token : FSIND       ; NParms : 1 ),
    ( ID : 'COSD'      ; Token : FCOSD       ; NParms : 1 ),
    ( ID : 'TAND'      ; Token : FTAND       ; NParms : 1 ),
    ( ID : 'ASIND'     ; Token : FASIND      ; NParms : 1 ),
    ( ID : 'ACOSD'     ; Token : FACOSD      ; NParms : 1 ),
    ( ID : 'ATAND'     ; Token : FATAND      ; NParms : 1 ),
    ( ID : 'SECD'      ; Token : FASECD      ; NParms : 1 ),
    ( ID : 'CSCD'      ; Token : FACSCD      ; NParms : 1 ),
    ( ID : 'COTD'      ; Token : FACOTD      ; NParms : 1 ),
    ( ID : 'ASECD'     ; Token : FASECD      ; NParms : 1 ),
    ( ID : 'ACSCD'     ; Token : FACSCD      ; NParms : 1 ),
    ( ID : 'ACOTD'     ; Token : FACOTD      ; NParms : 1 ),
    ( ID : 'CFRAME'    ; Token : FVCFRAME    ; NPARMS : 0 ),
    ( ID : 'CMAXFRAME' ; Token : FVCMAXFRAME ; NPARMS : 0 )
    );

Const
  { user defined data }
  MaxVarSym = 64;
  NumVarSym : Integer = 0;
  VarSym : Array[1..MaxVarSym] of Record  { variable symbol table }
    ID     : String[15];  { identifier name }
    Offset : Integer;     { offset of code for this data element.
                            < -1 for system variables }
    Valid  : Boolean;     { is the current function value valid? }
    Value  : Float        { value of this function }
  End = (
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0),
    (ID : '';       offset : -1; valid : FALSE; value : 0.0)
  );

Type
  PatchRecPtr = ^PatchRec;
  PatchRec = Record
    TLabel   : String[15];  { Label name }
    JumpAddr : Integer;     { address in code of label }
    Next     : PatchRecPtr  { next patch record }
  End; { Record }

Const
  LabelList       : PatchRecPtr = Nil; { list of labels                     }
  PatchList       : PatchRecPtr = Nil; { list of jump instrs to be patched  }


Var
  Series : Array[0..9] of Record { data for all series }
    Start   : Float;  { starting index value }
    Step    : Float;  { index step value }
    Finish  : Float;  { ending index value }
    Index   : Float;  { current index value }
    Result  : Float;  { partial result of series evaluation }
  End;

Const
  ConstExpression : Boolean = False;   { looking for constant expressions?  }

{===========================================================================}

Procedure Parse(InStr : String; Var Error,Posn : Integer);
{
  Parse statement
}
Const
  CurSeries : Integer = 0; { current series being evaluated }

Var

  CPos         : Integer;     { current position in input string }
  Ch           : Char;        { current lookahead character }
  CurTok       : TokenType;   { current token type }
  Skip         : Boolean;     { skip over character? }

  I            : Integer;     { scratch variable }

  IndexRet     : Integer;     { return value for indices }
  ConstRet     : Float;       { return for constants }
  StringRet    : String;      { return for strings }

  {-------------------------------------------------------------------------}

  Procedure GetToken;

    {-----------------------------------------------------------------------}

    Procedure GetChar;
    {
      Return one character from the input stream
    }
    Begin
      If CPos < Length(InStr) then begin
        Inc(CPos);
        Ch := UpCase(Instr[CPos])
      End { If }
      Else
        Error := NeedMoreChars
    End; { Procedure GetChar }

    {-----------------------------------------------------------------------}

    Procedure GetIdent;
    {
      Get an identifier from the input stream
    }
    Begin

      { accumulate identifier }
      StringRet := '';
      While Ch in ['A'..'Z','0'..'9'] do begin
        StringRet := StringRet + Ch;
        GetChar
      End; { While }

      { check against system symbols }
      IndexRet := 0;
      While (IndexRet < NumSysSym) and (StringRet <> SysSym[IndexRet].ID) do
        Inc(IndexRet);

      { if found }
      If IndexRet < NumSysSym then
        CurTok := SysSym[IndexRet].Token

      Else begin

        { check against user symbols }
        IndexRet := 1;
        While (IndexRet <= NumVarSym) and (StringRet <> VarSym[IndexRet].ID) do
          Inc(IndexRet);

        { if found }
        If IndexRet <= NumVarSym then
          CurTok := USERVAR
        Else
          CurTok := IDENT

      End { Else }

    End; { Procedure GetIdent }

    {-----------------------------------------------------------------------}

    Procedure GetNumber;
    {
      Return numeric value from input stream.

      Cheap algorithm -- it uses the entire string and keeps removing
      characters from the end until there stops being an error.  This is a
      slow way to do it, but it keeps the code simple and ALWAYS gets the
      right number.
    }
    Var
      Code : Integer;
    Begin

      { get rest of current string }
      StringRet := Copy(InStr, CPos, 255);

      { attempt to do a conversion }
      Val(StringRet,ConstRet,Code);

      { if it didn't work, keep remove chars until done }
      While (Code <> 0) and (Length(StringRet) > 0) do begin

        { remove one character from the end of the string }
        Dec(StringRet[0]);

        { attempt to convert it again }
        Val(StringRet,ConstRet,Code)

      End; { While }

      { if there was a conversion error }
      If Code <> 0 then
        Error := BadNumber

      { valid! }
      Else begin

        { skip past number (almost, leave some for next GetChar) }
        CPos := CPos + Length(StringRet) - 1;

        { return what type it was }
        CurTok := NUMBER

      End { Else }

    End; { Procedure GetNumber }

    {-----------------------------------------------------------------------}

  Begin

    { make certain that error position points to start of offending code }
    Posn := CPos;

    { set up to skip character when we're done }
    Skip := True;

    { skip past leading white space }
    While (Ch = ' ') do
      GetChar;

    { determine current symbol }
    Case Ch of

      'A'..
      'Z' : Begin

              { don't skip a character after this is done }
              Skip := False;

              { extract and categorize symbol }
              GetIdent

            End;

      '0'..
      '9' : GetNumber;

      { number or equality operator }
      '.' : If InStr[CPos + 1] in ['A'..'Z'] then begin

              { accumulate logical identifier }
              StringRet := '';
              I := 1;
              While (InStr[CPos + I] <> '.') and (I < Length(InStr)) do begin
                StringRet := StringRet + InStr[CPos + I];
                Inc(I);
              End; { While }

              { find out which identifier }
              If StringRet = 'EQ' then
                CurTok := LOEQ
              Else If StringRet = 'NE' then
                CurTok := LONE
              Else If StringRet = 'GT' then
                CurTok := LOGT
              Else If StringRet = 'LT' then
                CurTok := LOLT
              Else If StringRet = 'GE' then
                CurTok := LOGE
              Else If StringRet = 'LE' then
                CurTok := LOLE
              Else If StringRet = 'OR' then
                CurTok := LCOR
              Else If StringRet = 'AND' then
                CurTok := LCAND
              Else If StringRet = 'XOR' then
                CurTok := LCXOR
              Else If StringRet = 'NOT' then
                CurTok := LCNOT
              Else If StringRet = 'IF' then
                CurTok := LIF
              Else If StringRet = 'ELSE' then
                CurTok := LELSE
              Else If StringRet = 'GOTO' then
                CurTok := LGOTO
              Else If StringRet = 'ADD' then
                CurTok := AOADD
              Else If StringRet = 'SUB' then
                CurTok := AOSUB
              Else If StringRet = 'MUL' then
                CurTok := AOMUL
              Else If StringRet = 'DIV' then
                CurTok := AODIV
              Else If StringRet = 'EXP' then
                CurTok := AOEXP

              { or none of them }
              Else
                StringRet := '';

              { if something actually found }
              If Length(StringRet) <> 0 then
                CPos := CPos + I
              Else
                GetNumber

            End { If }
            Else
              GetNumber;

      '+' : CurTok := AOADD;

      '-' : CurTok := AOSUB;

      '!' : If InStr[CPos + 1] = '=' then begin
              CurTok := LONE;
              GetChar
            End { If }
            Else
              CurTok := LCNOT;

      '*' : If InStr[CPos + 1] = '*' then begin
              CurTok := AOEXP;
              GetChar
            End { If }
            Else
              CurTok := AOMUL;

      '/' : If InStr[CPos + 1] = '=' then begin
              CurTok := LONE;
              GetChar
            End { If }
            Else
              CurTok := AODIV;

      '^' : CurTok := AOEXP;

      '(' : CurTok := LPAREN;
      ')' : CurTok := RPAREN;

      '&' : If InStr[CPos + 1] = '&' then begin
              CurTok := LCAND;
              GetChar
            End { If }
            Else
              CurTok := LCAND;

      '|' : If InStr[CPos + 1] = '|' then begin
              CurTok := LCOR;
              GetChar
            End { If }
            Else
              CurTok := LCOR;

      '=' : Case InStr[CPos + 1] of
              '=' : Begin CurTok := LOEQ; GetChar End;
              '>' : Begin CurTok := LOGE; GetChar End;
              '<' : Begin CurTok := LOLE; GetChar End
              Else        CurTok := LOEQ;
            End; { Case }


      ':' : If InStr[CPos + 1] = '=' then begin
              CurTok := ASSIGN;
              GetChar
            End { If }
            Else If InStr[CPos + 1] = ':' then begin
              CurTok := LLABEL;
              GetChar;           { skip second colon }
              GetToken;          { get label text }
              Skip := False;     { already skipped a character }
            End { If }
            Else
              CurTok := LELSE;

      '?' : CurTok := LIF;

      '>' : If InStr[CPos+1] = '=' then begin
              CurTok := LOGE;
              GetChar;
            End { If }
            Else
              CurTok := LOGT;

      '<' : If InStr[CPos+1] = '=' then begin
              CurTok := LOLE;
              GetChar;
            End { If }
            Else
              CurTok := LOLT;

      ';' : Begin
              CurTok := ENDOF;
              Skip := False
            End;

      ',' : CurTok := COMMA;

      Else
        Error := BadChar

    End; { Case }

    { skip past character if required and possible }
    If (Error = NoError) and Skip then
      GetChar

  End; { Procedure GetToken }

  {------------------------------------------------------------------------}

  Procedure GenCode(Opcode : Byte);
  {
    Add a character to the code space
  }
  Begin
    If CodeTop >= SizeOf(Code) then
      Error := CodeOverflow
    Else begin
      Inc(CodeTop);
      Code[CodeTop] := Opcode
    End { Else }
  End; { Procedure GenCode }

  {------------------------------------------------------------------------}

  Procedure ParseLabel;
  {
    Parse a label.
    Adds the label address to the list of labels.
  }
  Var
    P, Q : PatchRecPtr;
  Begin

    { Eat the LLABEL token }
    GetToken;

    { If the next token is a label candidate,  }
    If CurTok in [USERVAR, IDENT] then begin

      { allocate a new node for the patch list }
      New(P);

      { copy the data to the new record }
      P^.TLabel   := StringRet;
      P^.JumpAddr := CodeTop + 1;
      P^.Next     := Nil;

      { Add the record to the list if not already there }
      If PatchList = Nil then
        PatchList := P
      Else begin
        Q := PatchList;
        While (Q^.Next <> Nil) and (P^.TLabel <> Q^.TLabel) do
          Q := Q^.Next;
        If (Q^.Next = Nil) then
          Q^.Next := P
        Else begin
          Error := LabelRedefined;
          Dispose(P);
        End; { Else }
      End; { Else }

      { eat the dest name token }
      GetToken;

    End { If }
    Else
      Error := IdentExpected

  End; { Procedure ParseLabel }

  {------------------------------------------------------------------------}

  Procedure ParseGoto;
  {
    Parse a GOTO instruction.  Generates a JMP operation and empty space,
    then adds a record to the list of jumps that need patching.
  }
  Var
    P, Q : PatchRecPtr;
  Begin

    { Eat the GOTO token }
    GetToken;

    { If the next token is a label candidate,  }
    If CurTok in [USERVAR, IDENT] then begin

      { allocate a new node for the patch list }
      New(P);

      { copy the data to the new record }
      P^.TLabel   := StringRet;
      P^.JumpAddr := CodeTop + 1;
      P^.Next := Nil;

      { Add the record to the list }
      If PatchList = Nil then
        PatchList := P
      Else begin
        Q := PatchList;
        While Q^.Next <> Nil do
          Q := Q^.Next;
        Q^.Next := P;
      End; { Else }

      { generate the JMP instruction }
      GenCode(Byte(VMJMP));
      GenCode(255);
      GenCode(255);

      { eat the dest name token }
      GetToken;

    End { If }
    Else
      Error := IdentExpected

  End; { Procedure ParseGoto }

  {------------------------------------------------------------------------}

  {
    Technically, these routines don't have to be declared forward.  They
    could be simply nested inside each other and the resulting executable
    should be the same.  Unfortunately, doing so makes the source code flow
    off the right-hand edge of standard 80-column paper.  The "purity" of
    the code suffers a little but makes all of the routines fit on the
    page.
  }

  Procedure ParseExpr;      Forward;  { only call this routine }
  Procedure RelExpression;  Forward;
  Procedure NumExpression;  Forward;
  Procedure Term;           Forward;
  Procedure ExpFactor;      Forward;
  Procedure Factor;         Forward;
  Procedure ParseSysFunc;   Forward;

  {------------------------------------------------------------------------}

  Procedure ParseSysFunc;
  {
    Process a system function w/ possibly multiple parameters.
  }
  Var
    Op : TokenType;
    I  : Integer;
  Begin

    { remember the function index }
    Op := CurTok;

    { if the function takes a parameter }
    If (SysSym[IndexRet].NParms > 0) then begin

      { get another token }
      GetToken;

      { If it's not a left paren, return error }
      If CurTok = LPAREN then begin

        { remember the number of parms to get }
        I := SysSym[IndexRet].NParms;

        { flush the LPAREN }
        GetToken;

        { while not at the end of the parm list }
        While (I > 1) and (Error = NoError) do begin

          { get an expression }
          ParseExpr;

          { if the next token is a comma, }
          if (CurTok = COMMA) then begin

            { skip it }
            GetToken;

            { and decrement the count of expressions remaining }
            Dec(I);

          End { If }

          { there should have been a comma }
          Else
            Error := CommaExpected;

        End; { While }

        { get the last expression in the list }
        ParseExpr;

        { and then set up to perform the operation that started it all }
        GenCode(Byte(Op));

        { and now there should be a right paren }
        If CurTok <> RPAREN then
          Error := RParenExpected

      End { If }
      Else
        Error := LParenExpected;

    End { If }
    Else

      { generate code for zero-parm function }
      GenCode(Byte(Op));

  End; { Procedure ParseSysFunc }

  {------------------------------------------------------------------------}

  Procedure ParseIfStmt;
  {
    Process an if statement of the form

        e1 IF e2 [ELSE e3]

    This means:  If e1 is not = 0, execute e2 else execute e3.
    It should translate to one of the following (non-else or else):

       non-else       else
           e1           e1
           JZ a1        JZ a1
           e2           e2
       a1:              JMP a2
                    a1: e3
                    a2:

  }
  Var
    IfOffset,              { code offset for JZ }
    ElseOffset : Integer;  { code offset for JMP }
  Begin

    { skip the IF token }
    GetToken;

    { generate the JZ opcode  (with blank offset bytes) }
    GenCode(Byte(VMJZ));
    GenCode(255);
    GenCode(255);

    { remember where the jump data goes }
    IfOffset := CodeTop - 1;

    { get the statement to execute if the expression is true }
    ParseExpr;

    { if the next token is an ELSE }
    If CurTok = LELSE then begin

      { eat the ELSE token }

      { generate the JMP around the ELSE code (with blank offset bytes) }
      GenCode(Byte(VMJMP));
      GenCode(255);
      GenCode(255);

      { remember where the jump data goes }
      ElseOffset := CodeTop - 1;

      { patch the JZ to the current address }
      Code[IfOffset + 0] := (CodeTop + 1) and 255;
      Code[IfOffset + 1] := (CodeTop + 1) shr 8;

      { get the expression to execute if the original consdition was FALSE }
      ParseExpr;

      { patch the JMP to the current address }
      Code[ElseOffset + 0] := (CodeTop + 1) and 255;
      Code[ElseOffset + 1] := (CodeTop + 1) shr 8;

    End { If }
    Else begin

      { patch the JZ to the current address }
      Code[IfOffset + 0] := (CodeTop + 1) and 255;
      Code[IfOffset + 1] := (CodeTop + 1) shr 8;

    End; { Else }

  End; { Procedure ParseIfStmt }

  {------------------------------------------------------------------------}

  Procedure ParseSeries(serSTART, serCONT : TokenType);
  {
    Process a series of the form

      seriesop ( start , step , finish , function )

    This means:

      evaluate start
      evaluate step
      evaluate finish
      init series accumulator
      if step < 0
        set index to finish
        while index >= start do
          evaluate function
          accumulate result
          increment index by step
        end while
      else
        set index to start
        while index <= finish do
          evaluate function
          accumulate result
          increment index by step
        end while


    should translate to:

         start code
         step code
         finish code
         serSTART
      a: function code
         serCONT label

  }
  Var
    ContAddr : Integer;   { address of eval expression start }
  Begin

    { skip the series start token }
    GetToken;

    { the next token should be a left parenthesis }
    If CurTok = LParen then begin

      GetToken;

      { increment the series number }
      Inc(CurSeries);

      { If still room for series }
      If CurSeries <= 9 then begin

        { generate the serSTART opcode w/series ID and blank offset }
        GenCode(Byte(serSTART));
        GenCode(Byte(CurSeries));

        { get start expression }
        ParseExpr;

        { a comma }
        If CurTok = Comma then begin
          GetToken;

          { step expression }
          ParseExpr;

          { a comma }
          If CurTok = Comma then begin
            GetToken;

            { finish expression }
            ParseExpr;

            { a comma }
            If CurTok = Comma then begin
              GetToken;

              { Remember where the evaluation expression starts }
              ContAddr := CodeTop + 1;

              { the expression to be seriesed }
              ParseExpr;

              { a final right parenthesis }
              If CurTok = RParen then begin

                { generate the serCONT operation (w/index, etc.) }
                GenCode(Byte(serCONT));
                GenCode(Byte(CurSeries));
                GenCode(255);
                GenCode(255);

                { patch the serCONT jump }
                Code[CodeTop - 1] := (ContAddr + 2) and 255;
                Code[CodeTop - 0] := (ContAddr + 2) shr 8;

              End { If }
              Else
                Error := RParenExpected;

            End { If }
            Else
              Error := CommaExpected;

          End { If }
          Else
            Error := CommaExpected;

        End { If }
        Else
          Error := CommaExpected;

        { now one less nested series }
        CurSeries := CurSeries - 1;

      End { If }
      Else
        Abort('Too many nested series');

    End { If }
    Else
      Error := LParenExpected;

  End; { Procedure ParseSeries }

  {------------------------------------------------------------------------}

  Function SeriesOp(Mask, S : String; Op : TokenType) : Boolean;
  {
    Determines if mask is the first sequence of characters in S.  If so
    and a series is currently in operation, parses off the index number
    at the end of the identifier and then generates Op for that series.
    Returns FALSE if a series operator was handled, True if not found.
  }
  Var
    SeriesNum : Integer;  { Series number for this operation }
  Begin

    { assume we won't find what we're looking for }
    SeriesOp := True;

    { if there is a series active }
    If CurSeries >= 0 then begin

      { if the mask is the first thing in the string }
      If Pos(Mask, S) = 1 then begin

        { assume we will be using the current index (0 offset) }
        SeriesNum := 0;

        { if there is a character after the mask, }
        If Length(Mask) <> Length(S) then begin

          { if the character is a digit }
          If S[Length(Mask)] in ['0'..'9'] then
            SeriesNum := Ord(S[Length(Mask)]) - Ord('0');

        End; { If }

        { if the desired series is one of those currently used }
        If SeriesNum <= CurSeries then begin

          { generate the opcode }
          GenCode(Byte(Op));

          { generate the series number relative to the current series }
          GenCode(Byte(CurSeries - SeriesNum));

        End { If }
        Else
          Error := SeriesNotDeepEnough;

      End; { If }

    End; { If }

  End; { Function SeriesOp }

  {------------------------------------------------------------------------}

  Procedure Factor;
  {
    Get a fundamental factor (ident, number, function, etc.
  }

  Var
    HaveNot : Boolean;

  Begin

    { no not yet }
    HaveNot := False;

    { check for unary NOT }
    If CurTok = LCNOT then begin

      { remember that we have one }
      HaveNot := True;

      { discard the token }
      GetToken

    End; { If }

    Case CurTok of

      NUMBER :
        Begin

          { gen opcode }
          GenCode(Byte(LOADCONST));

          { if still room in code space }
          If CodeTop + Sizeof(Float) + 1 < Sizeof(Code) then begin

            { copy the constant to the code space }
            Move(ConstRet, Code[CodeTop + 1], Sizeof(Float));

            { update the code top }
            CodeTop := CodeTop + Sizeof(Float);

          End { If }

          { no room, code overflow }
          Else
            Error := CodeOverflow;
        End;

      USERVAR :
        Begin
          GenCode(Byte(LOADVAR));
          GenCode(Byte(IndexRet));
        End;

      FSIN,      FCOS,      FTAN,      FASIN,     FACOS,     FATAN,
      FSEC,      FCSC,      FCOT,      FASEC,     FACSC,     FACOT,
      FSINH,     FCOSH,     FTANH,     FASINH,    FACOSH,    FATANH,
      FSECH,     FCSCH,     FCOTH,     FASECH,    FACSCH,    FACOTH,
      FSIND,     FCOSD,     FTAND,     FASIND,    FACOSD,    FATAND,
      FSECD,     FCSCD,     FCOTD,     FASECD,    FACSCD,    FACOTD,
      FEXP,      FEXP10,
      FLN,       FLOG,
      FSQR,      FSQRT,     FCBRT,
      FABS,
      FROUND,    FFLOOR,    FCEIL,     FFRAC,
      FFACT,
      FATAN2,
      FSINC,
      FBESSEL,
      FGAMMA,
      FODD,
      FSIGN,
      FDIST,
      FANGLE,
      FQUADFRAC,
      FDECIBEL,
      FSURFACE, FISURFACE :
        ParseSysFunc;

      FVANIMTIME, FVANIMSTART, FVANIMSTOP,
      FVCFRAME, FVCMAXFRAME,
      FVMANDEL, FVJULIA,
      FVX,      FVY,
      FVIX,     FVIY,
      FVR,      FVT,
      FVXMIN,   FVYMIN,
      FVXMAX,   FVYMAX,
      FVIXMAX,  FVIYMAX,
      FVRND,
      FVPI,
      FVE :
        GenCode(Byte(CurTok));

      FVSSUM  : ParseSeries(ADDSTART, ADDCONT);
      FVSPROD : ParseSeries(MULSTART, MULCONT);

      LIF :
        ParseIfStmt;

      LLABEL :
        ParseLabel;

      LGOTO :
        If ConstExpression then
          Error := NotInConst
        Else
          ParseGoto;

      ASSIGN :
        Error := InvalidAssign;

      LPAREN :
        Begin
          GetToken;
          ParseExpr;
          If CurTok <> RPAREN then
            Error := RParenExpected;
        End;

      RPAREN :
        Error := TooManyRParens;

      { unknown identifier }
      IDENT :

        { make the required peculiar checks for series operators }
        If      SeriesOp('START',  StringRet, LSTART ) then
        Else If SeriesOp('STEP',   StringRet, LSTEP  ) then
        Else If SeriesOp('FINISH', StringRet, LFINISH) then
        Else If SeriesOp('INDEX',  StringRet, LINDEX ) then
        Else If SeriesOp('RESULT', StringRet, LRESULT) then
        Else
          Error := UnknownID;

      { just plain unknown }
      Else
        Error := InvalidToken

    End; { Case }

    { push the not operator if required }
    If HaveNot then
      GenCode(Byte(LCNOT));

    { pick up next token from input stream of no reason not to }
    If Error = NoError then
      GetToken;

    { check for ! (factorial) }
    If (CurTok = LCNOT) and (Error = NoError) then begin

      { skip the token }
      GetToken;

      { and generate the factorial function }
      GenCode(Byte(FFACT));

    End; { If }

  End; { Procedure Factor }

  {------------------------------------------------------------------------}

  Procedure ExpFactor;
  {
    Process a factor composed of possible exponentiation operation
  }
  Begin

    { get a factor }
    Factor;

    { While the next token after the term is an ^, get another factor }
    While CurTok = AOEXP do begin

      { lose the token }
      GetToken;

      { get another factor }
      Factor;

      { if no error, generate the saved operator }
      If Error = NoError then
        GenCode(Byte(AOEXP));

    End; { While }

  End; { Procedure ExpFactor }

  {------------------------------------------------------------------------}

  Procedure Term;
  {
    Process a Term
  }
  Var
    Op : TokenType;

  Begin

    { get a factor }
    ExpFactor;

    { While the next token after the term is a * or /, get another term }
    While CurTok in [AOMUL, AODIV] do begin

      { save the token }
      Op := CurTok;

      { flush the token }
      GetToken;

      { get another factor }
      ExpFactor;

      { if no error, generate the saved operator }
      If Error = NoError then
        GenCode(Byte(Op));

    End; { While }

  End; { Procedure Term }

  {----------------------------------------------------------------------}

  Procedure NumExpression;
  {
    Process numeric expression (possible sign + expression)
  }
  Var
    Op : TokenType;
    Negative : Boolean;

  Begin

    { no negative yet }
    Negative := False;

    { check for unary sign }
    If CurTok in [AOSUB, AOADD] then begin

      { remember if the token was a minus sign }
      Negative := CurTok = AOSUB;

      { discard the sign }
      GetToken

    End; { If }

    { get a term }
    Term;

    { While the next token after the term is a + or -, get another term }
    While CurTok in [AOADD, AOSUB] do begin

      { save the token }
      Op := CurTok;

      { flush the token }
      GetToken;

      { get another term }
      Term;

      { if no error, generate the saved operator }
      If Error = NoError then
        GenCode(Byte(Op));

    End; { While }

    { push the negation operator if required }
    If Negative then
      GenCode(Byte(AONEG));

  End; { Procedure NumExpression }

  {------------------------------------------------------------------------}

  Procedure RelExpression;
  {
    process relational expression (expression w/ >, <, <=, >=, =, <>
  }
  Var
    Op : TokenType;

  Begin

    { get a numeric expression }
    NumExpression;

    { while the next token after the term is a * or /, get another term }
    While CurTok in [LOLT, LOGT, LOLE, LOGE, LOEQ, LONE] do begin

      { save the token }
      Op := CurTok;

      { flush the token }
      GetToken;

      { get another nueric expression }
      NumExpression;

      { if no error, generate the saved operator }
      If Error = NoError then
        GenCode(Byte(Op));

    End; { While }

  End; { Procedure RelExpression }

  {------------------------------------------------------------------------}

  Procedure ParseExpr;
  {
    parse expression containing and, or, xor
  }
  Var
    Op : TokenType;
  Begin

    { get a relational expression }
    RelExpression;

    { while the next token after the term is an AND, OR, XOR get another term }
    While CurTok in [LCAND, LCOR, LCXOR] do begin

      { save the token }
      Op := CurTok;

      { flush the token }
      GetToken;

      { get another relational expression }
      RelExpression;

      { if no error, generate the saved operator }
      If Error = NoError then
        GenCode(Byte(Op));

    End; { While }

  End; { Procedure ParseExpr }

  {------------------------------------------------------------------------}

Var
  NewID : String[15];
  NewOffset : Integer;

Begin

  { add an end-of-string character }
  InStr := InStr + ';';

  { init scanner to start searching on the first call }
  Ch := ' ';
  CPos := 0;
  CurTok := IDENT;

  { no error found yet }
  Error := NoError;

  { save starting offset of this expression }
  NewOffset := CodeTop + 1;

  { pick up first token from input stream }
  GetToken;

  { if a new identifier }
  If CurTok = IDENT then

    If ConstExpression then
      Error := ConstExpected

    Else begin

      { save the ident }
      NewID := StringRet;

      { pick up next symbol }
      GetToken;

      { give error if not := }
      If CurTok <> ASSIGN then
        Error := AssignExpected

      Else

        { if not enough symbol space, give error }
        If NumVarSym >= MaxVarSym then
          Error := TooManyExprs

        Else begin

          { lose assignment operator }
          GetToken;

          { parse expression }
          ParseExpr;

          { only generate code and add to symbol table if no error }
          If Error = NoError then begin

            { allocate space for this symbol }
            Inc(NumVarSym);

            { save the ID and starting offset }
            With VarSym[NumVarSym] do begin
              ID := NewID;
              Offset := NewOffset
            End; { With }

            { actually assign the computed value to the variable }
            GenCode(Byte(ASSIGN));
            GenCode(Byte(NumVarSym));

            { make sure we stop evaluating at some point }
            GenCode(Byte(VMRET));

          End { If }

        End { Else }

    End { Else }

  { not a new id, main expression }
  Else begin

    { parse it out }
    ParseExpr;

    { if no error }
    If Error = NoError then begin

      { make sure we know when to stop }
      GenCode(Byte(VMRET));

      { if actually trying to generate permanent code and not defined yet }
      If not ConstExpression then begin

        { if no main expression has been defined }
        If (MainEntry = -1) then begin

          { remember where the main function comes in }
          MainEntry := NewOffset;

          { and pass back an "Error" that indicates a main expression }
          Error := NoAssignExpr;

        End { If }
        Else
          Error := MainRedefinition;

      End { If }


    End; { If }

  End; { Else }

  { If there's a real error }
  If (Error <> NoError) and (Error < LastError) then

    { dump the generated code }
    CodeTop := NewOffset - 1

End; { Procedure Parse }

{---------------------------------------------------------------------------}

Procedure BackPatch;
{
  Resolves all jumps pending (in PatchList).  Deletes the list as it
  processes.
}
Var
  P, Q : PatchRecPtr;
Begin

  While PatchList <> Nil do begin

    { remember which record we are currently working with }
    P := PatchList;

    { skip the list head along to the next entry }
    PatchList := PatchList^.Next;

    { search the label list for a corresponding label }
    Q := LabelList;
    While (Q^.Next <> Nil) and (P^.TLabel <> Q^.TLabel) do
      Q := Q^.Next;

    { if found }
    If (Q^.Next <> Nil) then begin

      { patch the address in the jump instruction to the destination found }
      Code[P^.JumpAddr + 0] := Q^.JumpAddr and 255;
      Code[P^.JumpAddr + 1] := Q^.JumpAddr shr 8;

    End; { If }

    { get rid of the backpatch list entry in any case }
    Dispose(P)

  End { While }

End; { Procedure BackPatch }

{---------------------------------------------------------------------------}

{ evaluate contents of interpreter buffer }
{ these variable were moved here because global var accesses are a little
  faster than local (stack-based) accesses. }

Var
  SP           : Integer;               { evaluation stack pointer }
  Stack        : Array[1..64] of Float; { evaluation stack }
  Temp,                                 { temp value for Eval }
  TOS,                                  { current Top-Of-Stack value }
  EvX, EvY,                             { X and Y evaluation values }
  EvIX, EvIY   : Float;                 { X and Y grid index values }

Function MainEvaluate(PC : Integer) : Float;
{
  Evaluate code starting at StartPC until VMRET encountered
}
Const
  Fudge = {$IFOPT N+} 1E-3000 {$ELSE} 1E-20 {$ENDIF};
  ln10  = 2.302585092994045680;
Var
  JAddr : Integer;  { address calculated in jump instructions }
  Flag : Boolean;   { flag calculated in conditional jumps }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Procedure Push(Data : Float);
  {
    Pushes TOS onto the stack.
  }
  Begin
    Inc(SP);
    Stack[SP] := TOS;
    TOS := Data
  End; { Procedure Push }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

  Procedure InvalidOperation(Msg : String);
  {
    Handles a math function error as required.
  }
  Begin
    If MathCheck then
      Abort(Msg)
    Else
      TOS := MathErrValue
  End; { Procedure InvalidOperation }

  {- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -}

Begin

  { if still patches to make, make them }
  If PatchList <> Nil then
    BackPatch;

  { go until done }
  While Code[PC] <> Byte(VMRET) do begin

    Case TokenType(Code[PC]) of

      AOMUL :
        Begin
          TOS := Stack[SP] * TOS;
          SP := SP - 1
        End;

      AODIV :
        Begin
          If TOS = 0 then
            InvalidOperation('Divide by zero')
          Else
            TOS := Stack[SP] / TOS;
          SP := SP - 1
        End;

      AOSUB :
        Begin
          TOS := Stack[SP] - TOS;
          SP := SP - 1;
        End;

      AOADD  :
        Begin
          TOS := Stack[SP] + TOS;
          SP := SP - 1
        End;

      AOEXP :
        Begin { a^x }
          TOS := power(Stack[SP], TOS);
          SP := SP - 1
        End;

      AONEG :            { negate }
        TOS := -TOS;

      FVX         : Push(EvX);
      FVY         : Push(EvY);
      FVR         : Push(sqrt(sqr(EvX) + sqr(EvY)));
      FVT         : Push(atan2(EvY, EvX));
      FVIX        : Push(EvIX);
      FVIY        : Push(EvIY);
      FVIXMAX     : Push(XGridSize);
      FVIYMAX     : Push(YGridSize);
      FVXMIN      : Push(XMin);
      FVXMAX      : Push(XMax);
      FVYMIN      : Push(YMin);
      FVYMAX      : Push(YMax);
      FVMANDEL    : Push(Potential(0.0, 0.0, EvX,           EvY          ));
      FVJULIA     : Push(Potential(EvX, EvY, RealJuliaParm, ImagJuliaParm));
      FVRND       : Push(Random);
      FVPI        : Push(3.14159265358979323846);
      FVE         : Push(2.71828182845904523536);
      FVVIEWX     : Push(ViewPoint.X);
      FVVIEWY     : Push(ViewPoint.Y);
      FVVIEWZ     : Push(ViewPoint.Z);
      FVVIEWAZ    : Push(atan2(ViewPoint.Y, ViewPoint.X));
      FVVIEWEL    : Push(atan2(ViewPoint.Z, sqrt(sqr(ViewPoint.X) +
                                                 sqr(ViewPoint.Y) +
                                                 sqr(ViewPoint.Z))));
      FVVIEWR     : Push(sqrt(sqr(ViewPoint.X) +
                              sqr(ViewPoint.Y) +
                              sqr(ViewPoint.Z)));
      FVFOCUSX    : Push(Focus.X);
      FVFOCUSY    : Push(Focus.X);
      FVFOCUSZ    : Push(Focus.X);
      FVFOCUSAZ   : Push(atan2(Focus.Y, Focus.X));
      FVFOCUSEL   : Push(atan2(Focus.Z, sqrt(sqr(Focus.X) +
                                             sqr(Focus.Y) +
                                             sqr(Focus.Z))));
      FVFOCUSR    : Push(sqrt(sqr(Focus.X) + sqr(Focus.Y) + sqr(Focus.Z)));
      FVANIMTIME  : Push(AnimParm);
      FVANIMSTART : Push(AnimStart);
      FVANIMSTOP  : Push(AnimStop);

      FVCFRAME    : Push(CalcFrame); 
      FVCMAXFRAME : Push(MaxCalcFrames);

      LOEQ, LONE,
      LOLT, LOGT, LOLE, LOGE,
      LCAND, LCOR, LCXOR,
      LCNOT :
        Begin
          Case TokenType(Code[PC]) of
            LOEQ  : Flag := Stack[SP] =  TOS;
            LONE  : Flag := Stack[SP] <> TOS;
            LOLT  : Flag := Stack[SP] <  TOS;
            LOGT  : Flag := Stack[SP] >  TOS;
            LOLE  : Flag := Stack[SP] <= TOS;
            LOGE  : Flag := Stack[SP] >= TOS;
            LCAND : Flag := (Stack[SP] <> 0.0) and (TOS <> 0.0);
            LCOR  : Flag := (Stack[SP] <> 0.0) or  (TOS <> 0.0);
            LCXOR : Flag := (Stack[SP] <> 0.0) xor (TOS <> 0.0);
            LCNOT : Flag := Stack[SP] = 0.0;
          End; { Case }
          If Flag then
            TOS := 1.0
          Else
            TOS := 0.0;
          SP := SP - 1;
        End;


      VMJMP,  { jump to address }
      VMJZ,   { jump to address if TOS = 0 - pops TOS }
      VMJLZ,  { jump to address if TOS < 0 - pops TOS }
      VMJGZ : { jump to address if TOS > 0 - pops TOS }
        Begin
          JAddr := Code[PC + 1] + (Code[PC + 2] shl 8);
          If (JAddr >= 0) and (JAddr < Sizeof(Code)) then begin
            Case TokenType(Code[PC]) of
              VMJMP : Flag := True;
              VMJZ  : Flag := TOS = 0.0;
              VMJLZ : Flag := TOS < 0.0;
              VMJGZ : Flag := TOS > 0.0;
              Else    Flag := False;
            End; { Case }
            If Flag then
              PC := JAddr
            Else
              PC := PC + 2;
          End; { If }
          TOS := Stack[SP];
          SP := SP - 1;
        End;


      VMJEQ,  { jump to address if TOS-1 = TOS  - pops TOS and TOS - 1 }
      VMJLT,  { jump to address if TOS-1 < TOS  - pops TOS and TOS - 1 }
      VMJGT,  { jump to address if TOS-1 > TOS  - pops TOS and TOS - 1 }
      VMJLE,  { jump to address if TOS-1 <= TOS - pops TOS and TOS - 1 }
      VMJGE : { jump to address if TOS-1 >= TOS - pops TOS and TOS - 1 }
        Begin
          JAddr := Code[PC + 1] + (Code[PC + 2] shl 8);
          If (JAddr >= 0) and (JAddr < Sizeof(Code)) then begin
            Case TokenType(Code[PC]) of
              VMJEQ : Flag := Stack[SP] = TOS;
              VMJLT : Flag := Stack[SP] < TOS;
              VMJGT : Flag := Stack[SP] > TOS;
              VMJLE : Flag := Stack[SP] <= TOS;
              VMJGE : Flag := Stack[SP] >= TOS;
              Else    Flag := False;
            End; { Case }
            If Flag then
              PC := JAddr
            Else
              PC := PC + 2;
          End; { If }
          TOS := Stack[SP - 1];
          SP := SP - 2;
        End;

      { load constant from code space }
      LOADCONST :
        Begin

          { stuff TOS onto stack }
          Inc(SP);
          Stack[SP] := TOS;

          { copy number from code space to TOS }
          Move(Code[PC + 1], TOS, Sizeof(Float));

          { skip PC past the number in the code stream }
          PC := PC + Sizeof(Float);

        End;

      { load identifier value from table }
      LOADVAR :
        { use correct symbol number }
        With VarSym[Code[PC + 1]] do begin

          { if symbol has been evaluated }
          If valid then

            { just push the result }
            Push(value)

          Else begin

            { evaluate the other beastie }
            If MainEvaluate(Offset) = 0.0 then ;

            { it's now valid in the list }
            valid := true;

            { push the result }
            Push(value)

          End; { Else }

          { move the PC past the operand }
          Inc(PC);

        End; { With }


      { store TOS to identifier table }
      ASSIGN :
        { use correct symbol number }
        With VarSym[Code[PC + 1]] do begin

          { store the value }
          value := TOS;

          { it now is valid }
          valid := True;

          { pop the next value into TOS }
          TOS := Stack[SP];
          Dec(SP);

          { move the PC past the operand }
          Inc(PC);

        End; { With }


      FSIN :   { sin }
        TOS := sin(TOS);

      FCOS :   { cos }
        TOS := cos(TOS);

      FTAN :   { tan }
        Begin
          Temp := Cos(TOS);
          If Temp = 0.0 then
            InvalidOperation('Tangent singularity')
          Else
            TOS := sin(TOS) / Temp
        End;

      FASIN :   { arcsin }
        If abs(TOS) >= 1.0 then
          InvalidOperation('Arcsine out-of-bounds')
        Else
          TOS := arctan(TOS / sqrt(1 - sqr(TOS)));

      FACOS :   { arccos }
        If abs(TOS) >= 1.0 then
          InvalidOperation('Arccosine out-of-bounds')
        Else
          TOS := arctan(1 / sqrt(1 - sqr(TOS)));

      FATAN :   { arctan }
        TOS := arctan(TOS);

      FSEC :   { secant }
        Begin
          Temp := cos(TOS);
          If Temp = 0 then
            InvalidOperation('Secant singularity')
          Else
            TOS := 1.0 / Temp;
        End;

      FCSC :   { cosecant }
        Begin
          Temp := sin(TOS);
          If Temp = 0 then
            InvalidOperation('Cosecant singularity')
          Else
            TOS := 1.0 / Temp;
        End;

      FCOT : { cotangent }
        Begin
          Temp := sin(TOS);
          If Temp = 0 then
            InvalidOperation('Cotangent singularity')
          Else
            TOS := cos(TOS) / Temp;
        End;

      FASEC :   { arcsecant }
        If abs(TOS) < 1.0 then
          InvalidOperation('Arcsecant out-of-bounds')
        Else begin
          TOS := 1.0 / TOS;
          TOS := arctan(1 / sqrt(1 - sqr(TOS)));
        End; { Else }

      FACSC :   { arccosecant }
        If abs(TOS) < 1.0 then
          InvalidOperation('Arccosecant out-of-bounds')
        Else begin
          TOS := 1.0 / TOS;
          TOS := arctan(TOS / sqrt(1 - sqr(TOS)));
        End; { Else }

      FACOT :   { arccotangent }
        TOS := Pi / 2.0 - arctan(TOS);


      FSIND :   { degree sin }
        TOS := sin(TOS * Pi / 180.0);

      FCOSD :   { degree cos }
        TOS := cos(TOS * Pi / 180.0);

      FTAND :   { degree tan }
        Begin
          Temp := Cos(TOS * Pi / 180.0);
          If Temp = 0.0 then
            InvalidOperation('Tangent singularity')
          Else
            TOS := sin(TOS * Pi / 180.0) / Temp
        End;

      FASIND :   { degree arcsin }
        If abs(TOS) >= 1.0 then
          InvalidOperation('Arcsine out-of-bounds')
        Else
          TOS := 180.0 / Pi * arctan(TOS / sqrt(1 - sqr(TOS)));

      FACOSD :   { degree arccos }
        If abs(TOS) >= 1.0 then
          InvalidOperation('Arccosine out-of-bounds')
        Else
          TOS := 180.0 / Pi * arctan(1 / sqrt(1 - sqr(TOS)));

      FATAND :   { degree arctan }
        TOS := arctan(TOS);

      FSECD :   { degree secant }
        Begin
          Temp := cos(TOS * Pi / 180.0);
          If Temp = 0 then
            InvalidOperation('Secant singularity')
          Else
            TOS := 1.0 / Temp;
        End;

      FCSCD :   { degree cosecant }
        Begin
          Temp := sin(TOS * Pi / 180.0);
          If Temp = 0 then
            InvalidOperation('Cosecant singularity')
          Else
            TOS := 1.0 / Temp;
        End;

      FCOTD : { degree cotangent }
        Begin
          Temp := sin(TOS * Pi / 180.0);
          If Temp = 0 then
            InvalidOperation('Cotangent singularity')
          Else
            TOS := cos(TOS * Pi / 180.0) / Temp;
        End;

      FASECD :   { degree arcsecant }
        If abs(TOS) < 1.0 then
          InvalidOperation('Arcsecant out-of-bounds')
        Else begin
          TOS := 1.0 / TOS;
          TOS := 180.0 / Pi * arctan(1 / sqrt(1 - sqr(TOS)));
        End; { Else }

      FACSCD :   { degree arccosecant }
        If abs(TOS) < 1.0 then
          InvalidOperation('Arccosecant out-of-bounds')
        Else begin
          TOS := 1.0 / TOS;
          TOS := 180.0 / Pi * arctan(TOS / sqrt(1 - sqr(TOS)));
        End; { Else }

      FACOTD :   { degree arccotangent }
        TOS := 180.0 / Pi * (Pi / 2.0 - arctan(TOS));

      FSINH :   { hyperbolic sin }
        TOS := 0.5 * (exp(TOS) - exp(-TOS));

      FCOSH :   { hyperbolic cos }
        TOS := 0.5 * (exp(TOS) + exp(-TOS));

      FTANH :   { hyperbolic tan }
        Begin
          Temp := 0.5 * (exp(TOS) + exp(-TOS));
          If Temp = 0 then
            InvalidOperation('Hyperbolic tangent singularity')
          Else
            TOS := (0.5 * (exp(TOS) - exp(-TOS))) / Temp
        End;

      FASINH :   { hyperbolic arcsin }
        TOS := ln(TOS + sqrt(sqr(TOS) + 1));

      FACOSH :   { hyperbolic arccos }
        If TOS < 1.0 then
          InvalidOperation('Hyperbolic arccosine value out-of-bounds')
        Else
          TOS := ln(TOS + sqrt(sqr(TOS) - 1));

      FATANH :   { hyperbolic arctan }
        If abs(TOS) > 1.0 then
          InvalidOperation('Hyperbolic arctangent value out-of-bounds')
        Else
          TOS := 0.5 * ln((1 + TOS) / (1 - TOS));

      FSECH :   { hyperbolic secant }
        TOS := 2.0 / (exp(TOS) + exp(-TOS));

      FCSCH :   { hyperbolic cosecant }
        If TOS = 0 then
          InvalidOperation('Hyperbolic cosecant singularity')
        Else
          TOS := 2.0 / (exp(TOS) - exp(-TOS));

      FCOTH :   { hyperbolic cotangent }
        If TOS = 0 then
          InvalidOperation('Hyperbolic cotangent singularity')
        Else
          TOS := (exp(TOS) + exp(-TOS)) / (exp(TOS) - exp(-TOS));

      FASECH :   { hyperbolic arcsecant }
        If abs(TOS) >= 1.0 then
          InvalidOperation('Hyperbolic arcsecant value out-of-bounds')
        Else
          TOS := ln((1 + sqrt(1 - sqr(TOS))) / TOS);

      FACSCH :   { hyperbolic arccosecant }
        If TOS = 0 then
          InvalidOperation('Hyperbolic arccosecant singularity')
        Else
          TOS := ln((1/TOS) + (sqrt(1 + sqr(TOS))/abs(TOS)));

      FACOTH :   { hyperbolic arccotangent }
        If abs(TOS) >= 1 then
          InvalidOperation('Hyperbolic arctangent out-of-bounds')
        Else
          TOS := 0.5 * ln((TOS + 1) / (TOS - 1));

      FEXP :   { e^x }
        TOS := exp(TOS);

      FEXP10 :   { 10^x }
        TOS := exp(TOS * ln(10));

      FLN :   { natural log }
        If TOS <= 0.0 then
          InvalidOperation('Natural log <= 0')
        Else
          TOS := ln(TOS);

      FLOG :   { common log }
        If TOS <= 0.0 then
          InvalidOperation('Common log <= 0')
        Else
          TOS := ln(TOS) / ln10;

      FSQRT :   { square root }
        If TOS < 0 then
          InvalidOperation('Square root < 0')
        Else
          TOS := sqrt(TOS);

      FCBRT : { cube root }
        If TOS < 0 then
          InvalidOperation('Cube root < 0')
        Else
          TOS := exp(ln(TOS) / 3.0);

      FSQR :   { squared }
        TOS := sqr(TOS);

      FABS :   { absolute value }
        TOS := abs(TOS);

      FROUND :   { round nearest }
        TOS := int(TOS + 0.49999999999999);

      FFLOOR :   { round down }
        TOS := int(TOS);

      FCEIL :   { round up }
        TOS := int(TOS + 0.5);

      FFACT :   { factorial }
        TOS := Factorial(TOS);

      FFRAC :   { fractional part }
        TOS := Frac(TOS);

      FATAN2 :   { two-part arctangent function }
        Begin  { atan2(   Y     , X }
          TOS := atan2(Stack[SP], TOS);
          SP := SP - 1
        End;

      FSINC :   { sinc(TOS) == sin(TOS)/TOS }
        If TOS = 0.0 then
          TOS := 1.0
        Else
          TOS := sin(TOS) / TOS;

      FBESSEL :   { Bessel function }
        Begin
          TOS := Bessel(Stack[SP], TOS);
          SP := SP - 1;
        End;

      FDECIBEL : { convert input to decibels (dB) }
        If TOS <= 0.0 then
          InvalidOperation('dB conversion <= 0')
        Else
          TOS := ln(TOS) / ln10 * 10.0;

      FGAMMA :   { Gamma function }
        TOS := Gamma(TOS);

      FODD :   { odd function }
        TOS := Round(TOS) and 1;

      FSIGN :   { sign function }
        If TOS < 0 then
          TOS := -1
        Else If TOS > 0 then
          TOS := 1
        Else
          TOS := 0;

      FDIST :   { dist function }
        Begin
          TOS := sqrt(sqr(EvY - TOS) + sqr(EvX - Stack[SP]));
          SP := SP - 1;
        End;

      FANGLE :   { angle function }
        Begin
          TOS := atan2(EvY - Stack[SP], EvX - TOS);
          SP := SP - 1;
        End;


      FQUADFRAC :   { QUADFRAC }
        If SP >= 3 then begin
          TOS := Potential(Stack[SP-2], Stack[SP-1], Stack[SP], TOS);
          SP := SP - 3
        End { If }
        Else
          SP := 0;

      FSURFACE :
        (* currently doesn't do anything; just returns 0 *)
        Begin
          TOS := 0;
          SP := SP - 1;
        End;

      FISURFACE :

        { if precomputed and point is on surface }
        If Precompute and
           (Stack[SP] >= 0) and (Stack[SP] <= XGridSize) and
           (TOS       >= 0) and (TOS       <= YGridSize) then begin

          { set TOS to the point }
          TOS := Grid[Trunc(Stack[SP])]^[Trunc(TOS)];

          { discard stack bias }
          SP := SP - 1;

        End { If }
        Else begin
          TOS := 0;
          SP := SP - 1;
        End;  { Else }


      ADDSTART,  { sets up summation series }
      MULSTART : { sets up product series }

        If (Code[PC + 1] < 0) or (Code[PC + 1] > 9) then
          Abort('Series out of range')
        Else begin

          { set up the series specified }
          With Series[Code[PC + 1]] do begin

            { set up the series }
            Start  := Stack[SP - 1];
            Step   := Stack[SP];
            Finish := TOS;

            { make sure Start < Finish }
            If Start > Finish then begin
              TOS := Finish;
              Finish := Start;
              Start := TOS;
            End; { If }

            { dump the setup info from the stack }
            TOS := Stack[SP - 2];
            SP := SP - 3;

            { bomb out if step was zero }
            If Step = 0 then
              Abort('Attempt to perform series with step of zero');

            { set up starting index value }
            If Step < 0 then
              Index := Finish
            Else
              Index := Start;

            { set up accumulator }
            If TokenType(Code[PC]) = ADDSTART then
              Result := 0.0
            Else
              Result := 1.0;

          End; { With }

        End; { Else }


      ADDCONT,   { continues summation series }
      MULCONT  : { continues product series }
        If (Code[PC + 1] < 0) or (Code[PC + 1] > 9) then
          Abort('Series out of range')
        Else begin

          { use the series specified }
          With Series[Code[PC + 1]] do begin

            { accumulate result }
            If TokenType(Code[PC]) = ADDSTART then
              Result := Result + TOS
            Else
              Result := Result * TOS;

            { pop stack }
            TOS := Stack[SP];
            SP := SP - 1;

            { add the step to the index }
            Index := Index + Step;

            { compute the dest address and bound it }
            JAddr := Code[PC + 2] + (Code[PC + 3] shl 8);
            If (JAddr >= 0) and (JAddr < Sizeof(Code)) then begin

              { test for completion }
              If Step < 0 then
                Flag := Index < Start
              Else
                Flag := Index > Finish;

              { not done with series, go back and reevaluate }
              If Flag then
                PC := JAddr

              { done with series, move to next instr and push series result }
              Else begin
                PC := PC + 2;
                Push(Result);
              End; { Else }

            End; { If }

          End; { With }

        End; { Else }

      { series information operators }
      LSTART,
      LSTEP,
      LFINISH,
      LINDEX,
      LRESULT :
        If (Code[PC + 1] < 0) or (Code[PC + 1] > 9) then
          Abort('Series out of range')
        Else begin
          With Series[Code[PC + 1]] do
            Case TokenType(Code[PC]) of
              LSTART  : Push(Start);
              LSTEP   : Push(Step);
              LFINISH : Push(Finish);
              LINDEX  : Push(Index);
              LRESULT : Push(Result);
            End; { Case }
          PC := PC + 1;
        End; { Else }

    End; { Case }

    { inc program counter }
    Inc(PC);

  End; { While }

  { return result }
  MainEvaluate := TOS

End; { Function MainEvaluate }

{---------------------------------------------------------------------------}

Function IEvaluate(X, Y : Float; IX, IY : Integer) : Float;
{
  Calling shell for evaluate routine.  Sets up the integer index variables
  and then calls the main evaluate routine.
}
Begin

  { set up indices }
  EvIX := IX;
  EvIY := IY;

  { do the evaluation }
  IEvaluate := Evaluate(X, Y);

End; { Function IEvaluate }

{---------------------------------------------------------------------------}

Function Evaluate(X, Y : Float) : Float;
{
  Calling shell for evaluate routine.  Sets up the variable table and
  stack pointer, then evaluates the main function.
}
Var
  I : Integer;
Begin

  { check for error }
  If (MainEntry < 0) or (MainEntry > SizeOf(Code) div SizeOf(Code[0])) then
    Abort('no main expression has been located!')

  Else begin

    { clear valid flags for variables }
    For I := 1 to NumVarSym do
      VarSym[I].valid := False;

    { clear evaluation stack }
    SP := 0;

    { remember what we are evaluating the function for }
    EvX := X;
    EvY := Y;

    { compute function value at X,Y }
    Evaluate := MainEvaluate(MainEntry)

  End { Else }

End; { Function Evaluate }

{===========================================================================}

Function EvalConst(Expr : String) : Float;
{
  Evaluate a constant expression
}
Var
  I           : Integer;
  OldCodeTop  : Integer;
  Error       : Integer;
  Posn        : Integer;
  OK          : Boolean;
Begin

  { save the old code top }
  OldCodeTop := CodeTop;

  { tell the parser we're looking for constants }
  ConstExpression := True;

  { attempt to make something of this expression }
  Parse(Expr, Error, Posn);

  { assume the worst }
  EvalConst := ConstError;

  { if no error }
  If (Error = NoError) or (Error > LastError) then begin

    { clear valid flags for variables }
    For I := 1 to NumVarSym do
      VarSym[I].Valid := False;

    { clear evaluation stack }
    SP := 0;

    { compute value }
    EvalConst := MainEvaluate(OldCodeTop + 1)

  End; { If }

  { regardless of the function return value, clear the code space }
  CodeTop := OldCodeTop;

  { not looking for constants any more }
  ConstExpression := False;

End; { Function EvalConst }

{---------------------------------------------------------------------------}

Procedure Disassemble;
{
  Disassembles the code space.
}
Var
  PC    : Integer;      { current disassembly index }
  OldPC : Integer;      { previous PC (used for looking into the var table) }

  {-------------------------------------------------------------------------}

  Procedure ShowLiteral(Operator : String; Len : Integer);
  {
    Displays an arithmetic operator Operator, with a length Len.
  }
  Begin

    Write(Operator, ' ');
    PC := PC + Len;

  End; { Procedure ShowLiteral }

  {-------------------------------------------------------------------------}

  Function ShowToken(Token : TokenType; Len : Integer) : Boolean;
  {
    Displays the text of Token, preceded by Len bytes of code.  If the token
    was not found, returns False; if it was found, returns True.
  }
  Var
    I : Integer;
  Begin

    I := 0;
    While (I < NumSysSym) and (Token <> SysSym[I].Token) do
      I := I + 1;

    If (I < NumSysSym) then begin
      Write(SysSym[I].ID, ' ');
      PC := PC + Len;
      ShowToken := True;
    End  { If }
    Else
      ShowToken := False;

  End; { Function ShowToken }

  {-------------------------------------------------------------------------}

Var
  DV : Boolean;

Begin

  DV := DirectVideo;
  DirectVideo := False;

  { start at the beginning of the code space }
  PC := 0;

  { whie not done }
  While PC <= CodeTop do begin

    { remember where this instruction started }
    OldPC := PC;

    { output the line header }
    Writeln;
    Write(PC:4, ': ');

    { process the tokens }
    Case TokenType(Code[PC]) of

      AOMUL    : ShowLiteral('MUL', 1);
      AODIV    : ShowLiteral('DIV', 1);
      AOSUB    : ShowLiteral('SUB', 1);
      AOADD    : ShowLiteral('ADD', 1);
      AOEXP    : ShowLiteral('RAISE', 1);
      AONEG    : ShowLiteral('NEG', 1);
      LOEQ     : ShowLiteral('EQ',  1);
      LONE     : ShowLiteral('NE',  1);
      LOLT     : ShowLiteral('LT',  1);
      LOGT     : ShowLiteral('GT',  1);
      LOLE     : ShowLiteral('LE',  1);
      LOGE     : ShowLiteral('GE',  1);
      LCAND    : ShowLiteral('AND', 1);
      LCOR     : ShowLiteral('OR',  1);
      LCXOR    : ShowLiteral('XOR', 1);
      LCNOT    : ShowLiteral('NOT', 1);


      VMJMP,  VMJZ,   VMJLZ,  VMJGZ,
      VMJEQ,  VMJLT,  VMJGT,  VMJLE,
      VMJGE :
        Begin
          Case TokenType(Code[PC]) of
            VMJMP : Write('JMP');
            VMJZ  : Write('JZ ');
            VMJLZ : Write('JLZ');
            VMJGZ : Write('JGZ');
            VMJEQ : Write('JEQ');
            VMJLT : Write('JLT');
            VMJGT : Write('JGT');
            VMJLE : Write('JLE');
            VMJGE : Write('JGE');
          End; { Case }
          Write(' ', Code[PC + 1] + (Code[PC + 2] shl 8));
          PC := PC + 3;
        End;


      LOADCONST :
        Begin

          { copy number from code space to TOS }
          Move(Code[PC + 1], TOS, Sizeof(Float));

          Write('LOAD ', TOS);

          PC := PC + Sizeof(Float) + 1;

        End;

      LOADVAR :
        With VarSym[Code[PC + 1]] do begin
          Write('LOAD ''', VarSym[Code[PC + 1]].ID, '''');
          PC := PC + 2;
        End; { With }

      ASSIGN :
        With VarSym[Code[PC + 1]] do begin
          Write('STORE ''', VarSym[Code[PC + 1]].ID, '''');
          PC := PC + 2;
        End; { With }


      ADDSTART :
        Begin
          Write('ADDSTART ', Code[PC + 1]);
          PC := PC + 2;
        End;

      MULSTART :
        Begin
          Write('MULSTART ', Code[PC + 1]);
          PC := PC + 2;
        End;

      LSTART   :
        Begin
          Write('LOAD START ', Code[PC + 1]);
          PC := PC + 2;
        End;

      LSTEP    :
        Begin
          Write('LOAD STEP ', Code[PC + 1]);
          PC := PC + 2;
        End;

      LFINISH  :
        Begin
          Write('LOAD FINISH ', Code[PC + 1]);
          PC := PC + 2;
        End;

      LINDEX   :
        Begin
          Write('LOAD INDEX ', Code[PC + 1]);
          PC := PC + 2;
        End;

      LRESULT  :
        Begin
          Write('LOAD RESULT ', Code[PC + 1]);
          PC := PC + 2;
        End;

      ADDCONT :
        Begin
          Write('ADDCONT ',
                Code[PC + 1], ', ',
                Code[PC + 2] + (Code[PC + 3] shl 8));
          PC := PC + 4;
        End;

      MULCONT :
        Begin
          Write('MULCONT ',
                Code[PC + 1], ', ',
                Code[PC + 2] + (Code[PC + 3] shl 8));
          PC := PC + 4;
        End;

      VMRET :
        Begin
          Writeln('RETURN');
          PC := PC + 1;
        End;

      Else
       If not ShowToken(TokenType(Code[PC]), 1) then begin
         Write('Unknown: ', Code[PC]);
         PC := PC + 1;
       End; { If }

    End; { Case }

  End; { While }

  DirectVideo := DV;

End; { Disassemble }

{===========================================================================}

Function EvalErrMsg(Error : Integer) : String;
{
  Return a descriptive error message for the error number given
}
Const
  MsgTable : Array[0..LastError - 1] of String[25] = (
    { NoError          } 'No Error',
    { NeedMoreChars    } 'Need more characters',
    { BadChar          } 'Bad character',
    { BadNumber        } 'Invalid number',
    { InvalidToken     } 'Invalid token',
    { InvalidAssign    } 'Invalid assign',
    { UnknownID        } 'Unknown identifier',
    { OperatorExpected } 'Operator expected',
    { OperandExpected  } 'Operand expected',
    { AssignExpected   } ''':='' expected',
    { ConstExpected    } 'Constant expected',
    { CommaExpected    } ''','' expected',
    { TooManyConsts    } 'Too many constants',
    { TooManyExprs     } 'Too many expressions',
    { CodeOverflow     } 'Code overflow',
    { RParenExpected   } ''')'' expected',
    { LParenExpected   } '''('' expected',
    { TooManyRParens   } 'Too many '')''',
    { MainRedefinition } 'Main expression redefined',
    { IdentExpected    } 'Identifier Expected',
    { LabelRedefined   } 'Label Redefined',
    { NotInConst       } 'Not in Constant Expression',
    { SeriesNotDeepEnough } 'Series not nested deeply enough for this'
  );

Begin
  If (Error < 0) or (Error >= LastError) then
    EvalErrMsg := 'Unknown error code'
  Else
    EvalErrMsg := MsgTable[Error]
End; { Function EvalErrMsg }

{---------------------------------------------------------------------------}

Function MainFunctionDefined;
{
  Is the main function defined?
}
Begin
  MainFunctionDefined := MainEntry >= 0
End; { Function MainFunctionDefined }

{---------------------------------------------------------------------------}

Function GetVarValue(VarName : String) : Float;
{
  Evaluates the variable whose name is given in VarName.  If the variable is
  undefined, returns ConstError.
}
Var
  I : Integer;
Begin

  { convert the input string to upper case }
  For I := 1 to Length(VarName) do
    VarName[I] := UpCase(VarName[I]);

  { assume we won't find the name we're looking for in the list }
  GetVarValue := ConstError;

  { try to find the variable name in the symbol table }
  For I := 1 to NumVarSym do begin
    If (VarSym[I].ID = VarName) and (VarSym[I].Offset >= 0) then begin
      GetVarValue := MainEvaluate(VarSym[I].Offset);
    End; { If }
  End; { For }

End; { Function GetVarValue }

{---------------------------------------------------------------------------}

Begin

  { init start of code }
  CodeTop := -1;

  { clear the code space }
  FillChar(Code, SizeOf(Code), 0);

End. { Unit ParsEval }


{---------------------------------------------------------------------------}
