Unit Print;

Interface

Const

  Normal        = $00;  { mode settings for output (bit fields) }
  Inverse       = $01;

  PostScript    = 0;    { PostScript output }

  PrinterNames  : Array[0..2] of String[15] = (
    'PostScript', '', '');

  PrintMode     : Byte    = Normal;     { set default output mode }
  Printer       : Byte    = PostScript; { set default printer type }
  PrintFileName : String[128] = 'PRN';  { printer output file }

Procedure OpenPrinter;
Procedure PrintStr(S : String);
Procedure ClosePrinter;
Procedure ScreenPrint;
Procedure EjectPage;


Implementation

Uses
  CRT,
  Graph,
  FDecls;

Var
  PrintFile   : Text;     { PostScript output file }

{---------------------------------------------------------------------------}

Procedure OpenPrinter;
{
  Open printer file for output
}
Begin

  { attempt to open printer file }
  {$I-}
  Assign(PrintFile,PrintFileName);
  Rewrite(PrintFile);
  {$I+}

  { unable to open, bomb out of program }
  If IOResult <> 0 then
    Abort('Unable to open printer file');

End; { Procedure OpenPrinter }

{---------------------------------------------------------------------------}

Procedure ClosePrinter;
{
  Close the printer file when we're done with it
}
Begin
  {$I-}
  Close(PrintFile)
  {$I+}
End; { Procedure ClosePrinter }

{---------------------------------------------------------------------------}

{ send a string to the printer }
Procedure PrintStr;
Var
  LParen,RParen,
  I : Integer;
  ErrCount : Integer;
Begin

  ErrCount := 0;

  { find leftmost paren }
  LParen := 1;
  While (LParen <= Length(S)) and (S[LParen] <> '(') do
    Inc(LParen);

  { find rightmost paren }
  RParen := Length(S);
  While (RParen > 0) and (S[RParen] <> ')') do
    Dec(RParen);
  Dec(RParen);

  { process text between parens }
  While RParen > LParen do begin
    If (S[RParen] = '(') or (S[RParen] = ')') or (S[RParen] = '\') then
      Insert('\',S,RParen);
    Dec(RParen)
  End; { While }

  { output data, allowing for timeout problems }
  I := 1;
  Repeat
    {$I-}
    Write(PrintFile,S[I]);
    {$I+}
    If IOResult <> 0 then begin
      Inc(ErrCount);
      Delay(50)
    End { If }
    Else begin
      Inc(I);
      ErrCount := 0
    End { Else }
  Until (I > Length(S)) or (ErrCount > 100);

  If (IOResult <> 0) or (ErrCount <> 0) then
    Abort('Unable to write to printer')

End; { Procedure PrintStr }

{---------------------------------------------------------------------------}

Procedure ScreenPrint;
{
  Dumps the current screen image to a PostScript printer.
}
Const
  HexTable : Array[0..15] of Char = '0123456789ABCDEF';
Var
  LineBuf : Array[0..1023] of Char;
  OutBuf  : String[80];
  X, Y, I : Integer;
  C : LongInt;
Begin

  { output raster header }

  { start pixel count at 0 }
  C := 0;

  { for each scan line }
  For Y := 0 to PYMax do begin

    { read line from image }
    GetImage(0, PXMax, Y, Y, LineBuf);

    { set output string to empty }
    OutBuf := '';

    { for each pixel on the line }
    For X := 0 to PXMax do begin

      { one more pixel processed }
      Inc(C);

      { If the output count is divisible by 32, }
      If (X and 31) = 0 then begin

        { send output string and EOL }
        PrintStr(OutBuf + #13#10);

        { clear out the output buffer again }
        OutBuf := '';

      End; { If }

      { convert to hex in output buffer }
      OutBuf := OutBuf + HexTable[(Ord(LineBuf[X]) shr 4) and 15] +
                         HexTable[(Ord(LineBuf[X])      ) and 15];

    End; { For }

  End; { For }

  { output last buffer line to the printer. }
  PrintStr(OutBuf + #13#10);

End; { Procedure ScreenPrint }

{---------------------------------------------------------------------------}

Procedure EjectPage;
{
  Eject page from PostScript printer
}
Begin

  { feed out page from printer }
  If UsePostScript then begin

    { restore saved state }
    PrintStr(' grestore'#13#10);

    { dump page image if user quit }
    If Ch = #27 then
      PrintStr('erasepage')

    { else print the final information and spit out the page }
    Else begin

      { If actually showing this stuff }
      If not DrawInfo then begin

        { locate the right font }
        PrintStr('/Courier-Bold findfont 12 scalefont setfont'#13#10 +
                 'gsave'#13#10);
        PrintStr('-388 288 translate'#13#10 +
                 '/t { translate 0 0 moveto } def'#13#10);

        { show what function this is }
        If ShowFunc then
          PrintStr('0 -20 t (' + FuncStr + ') show'#13#10);

        { and over what range }
        If ShowRange then begin
          PrintStr('0 -12 t (' +
                   'X Range: ' + Float2Str(XMin,5) + '..' +
                                 Float2Str(XMax,5) + ') show'#13#10);
          PrintStr('0 -12 t (' +
                   'Y Range: ' + Float2Str(YMin,5) + '..' +
                                 Float2Str(YMax,5) + ') show'#13#10);
        End; { If }

        PrintStr('grestore'#13#10);

        { print title }
        PrintStr('(' + TitleString + ') dup stringwidth pop 2 div neg ' +
                 '-288 moveto show'#13#10);

      End; { If }

      { output page }
      PrintStr('showpage');

    End; { Else }

    { tell interpreter to stop now }
    PrintStr(' quit'#13#10);

    ClosePrinter

  End { If }

End; { Procedure EjectPage }

{-------------------------------------------------------------------------}


End. { Unit Print }
