Unit Triangle;

(****************************************************************************
**                                                                         **
**  Draw a patterned 2-D triangle on the screen                            **
**                                                                         **
****************************************************************************)

Interface

Uses
  FDecls;

Procedure FillTriangle(x1,y1, x2,y2, x3,y3 : Integer);
Procedure SetRGBFillPattern(Color : ColorType);
Procedure SetSolidFillPattern(Color : Byte);

Implementation

Uses
  Graph;

Type
  FillPatType = Array[0..7] of Array[0..7] of Byte;
  EdgeType = Array[0..1027] of Integer;

Var
  FillPat      : FillPatType;
  XLeft, XRight : ^EdgeType; { scan line edges of triangle }

{-------------------------------------------------------------------------}

Procedure FillTriangle;
{
  Fill triangle on screen
}
Var
  dX,dY,            { distance between line end points }
  dXAbs,dYAbs,      { # steps between line end points }
  I,J,K,II,         { loop counters and scratch }
  PX,PY,            { pixel XY coordinate }
  StepX,StepY,      { delta value to next pixel }
  XError,YError,    { Bresenham's running error totals }
  Top,Bottom    : Integer; { vertical extents of triangle }
  XPoint,YPoint : Array[0..3] of Integer;    { data points for line ends }

  { return sign of value, -1 0 +1 }
  Function Sign(X : Integer) : Integer;
  Begin
    If X = 0 then
      Sign := 0
    Else If X > 0 then
      Sign := +1
    Else
      Sign := -1
  End; { Function Sign }

  { fill a range of words }
  Procedure FillWord(Var Dest; Count,Data : Word);
  InLine($58/             { POP  AX                    }
         $59/             { POP  CX                    }
         $5F/             { POP  DI                    }
         $07/             { POP  ES                    }
         $FC/             { CLD                        }
         $F3/$AB);        { REP  STOSW                 }

Begin

  { set up data points in arrays }
  XPoint[0] := x1;
  YPoint[0] := y1;
  XPoint[1] := x2;
  YPoint[1] := y2;
  XPoint[2] := x3;
  YPoint[2] := y3;
  XPoint[3] := XPoint[0];
  YPoint[3] := YPoint[0];

  { determine span from top to bottom of triangle }
  If YPoint[0] < YPoint[1] then begin
    Top := YPoint[0];
    Bottom := YPoint[1]
  End { If }
  Else begin
    Top := YPoint[1];
    Bottom := YPoint[0]
  End; { Else }
  If YPoint[2] < Top then
    Top := YPoint[2];
  If YPoint[2] > Bottom then
    Bottom := YPoint[2];

  { if top and bottom are both off the screen in same direction, don't draw }
  If ((Top < 0) and (Bottom < 0)) or ((Top > PYMax) and (Bottom > PYMax)) then
    Exit;

  { make sure span fits on page }
  If Top < 1 then
    Top := 1;
  If Bottom >= PYMax then
    Bottom := PYMax - 1;

  { clear the span buckets for each line used }
  FillWord(XLeft^[Top - 1],  (Bottom - Top) + 2, PXMax);
  FillWord(XRight^[Top - 1], (Bottom - Top) + 2, 0);

  { go through each line }
  For J := 2 downto 0 do begin

    { start at a point on each line }
    PX := XPoint[J];
    PY := YPoint[J];

    { find size of line components }
    dX := XPoint[J + 1] - XPoint[J];
    dY := YPoint[J + 1] - YPoint[J];

    { and step factor in each direction }
    StepX := sign(dX);
    StepY := sign(dY);

    { find # steps to complete line in each direction }
    dXAbs := abs(dX);
    dYAbs := abs(dY);

    { init error values }
    XError := 0;
    YError := 0;

    { go in longer direction }
    If dXAbs > dYAbs then

      For K := dXAbs downto 0 do begin

        { "plot" pixel in scan buckets if possible }
        If (PY >= 0) and (PY <= PYMax) then begin
          If PX < XLeft^[PY] then
            XLeft^[PY] := PX;
          If PX > XRight^[PY] then
            XRight^[PY] := PX
        End; { If }

        { update error term and pixel position }
        YError := YError + dYAbs;
        PX := PX + StepX;

        { update other term and position if required }
        If YError >= dXAbs then begin
          YError := YError - dXAbs;
          PY := PY + StepY
        End; { If }

      End { For }

    Else

      For K := dYAbs downto 0 do begin

        { "plot" pixel in scan buckets if possible }
        If (PY >= 0) and (PY <= PYMax) then begin
          If PX < XLeft^[PY] then
            XLeft^[PY] := PX;
          If PX > XRight^[PY] then
            XRight^[PY] := PX
        End; { If }

        { update error term and pixel position }
        XError := XError + dXAbs;
        PY := PY + StepY;

        { update other term and position if required }
        If XError >= dYAbs then begin
          XError := XError - dYAbs;
          PX := PX + StepX
        End; { If }

      End; { For }

  End; { For }

  { make sure the triangle is onscreen }
  If (Top >= 0) and (Bottom > Top) and (Bottom <= PYMax) then begin

    { bound and draw each scan line }
    For I := Top to Bottom do begin

      { bound horizontally }
      If XLeft^[I] < 0 then
        XLeft^[I] := 0;
      If XRight^[I] > PXMax then
        XRight^[I] := PXMax;

      { remove an extra "I and 7" from the drawing loop }
      II := I and 7;

      { draw pixels using the color mask }
      For J := XLeft^[I] to XRight^[I] do
        PutPixel(J, I, FillPat[II][J and 7])

    End { For }

  End { If }

End; { Procedure FillTriangle }

{-------------------------------------------------------------------------}

Procedure SetRGBFillPattern;
{
  Set a fill pattern for the FillTriangle routine.  This routine uses
  a color ordered dither to set the fill pattern based on the current
  color[n] value
}
Var
  IC, I, C,
  X, Y, M,
  HOfs, VOfs : Integer;
Begin

  { clear fill pattern }
  HOfs := 0;
  VOfs := 0;

  FillChar(FillPat, SizeOf(FillPat), Black);

  For I := RColor to MaxComponent do begin

    Case I of
      RColor : Begin C := Red;              End;
      GColor : Begin C := Green; Inc(HOfs); End;
      BColor : Begin C := Blue;  Inc(VOfs); End;
    End; { Case }

    IC := Round(Color[I] * 64);

    For Y := 0 to 7 do begin
      M := $80;
      For X := 0 to 7 do begin
        If (Halftones[IC][Y] and M) <> 0 then
          FillPat[Y][X] := C;
        M := M shr 1;
      End; { For }
    End; { For }

  End; { For }

End; { Procedure SetRGBFillPattern }

{-------------------------------------------------------------------------}

Procedure SetSolidFillPattern;
{
  Set a solid fill pattern for the FillTriangle routine.
}
Begin
  FillChar(FillPat, SizeOf(FillPat), Color)
End; { Procedure SetSolidFillPattern }

{-------------------------------------------------------------------------}


Begin

  New(XLeft);
  New(XRight);

End.