Unit View;

(****************************************************************************
**                                                                         **
**                    3D viewing transformation stuff.                     **
**                                                                         **
****************************************************************************)

Interface

Uses
  Vectors;

Type
  { transform structure in project routine }
  Matrix = Array[1..4,1..4] of Float;

Const
  IdentMatrix : Matrix = (( 1, 0, 0, 0),  { identity matrix }
                          ( 0, 1, 0, 0),
                          ( 0, 0, 1, 0),
                          ( 0, 0, 0, 1));

Procedure SetScalingFactors(HScale, VScale : Float);
Procedure GetViewMat(Eye, Focus : Cartesian);
Function  Transform(x,y,z : Float; Var sx,sy : Single) : Boolean;
Procedure MultMat(Var m3, m1, m2 : Matrix);
Procedure SetDSRatio(DSR : Float);

Implementation

Const
  HorzScale    : Float = 1.0;  { scaling factors }
  VertScale    : Float = 1.0;
  DSRatio      : Float = 1.0;  { D/S ratio for 3D clipping, etc. }

Var
  ViewMat      : Matrix;      { current viewing transformation }

{---------------------------------------------------------------------------}

Procedure SetScalingFactors(HScale, VScale : Float);
{
  Set global scaling factors
}
Begin
  HorzScale := HScale;
  VertScale := VScale
End; { Procedure SetScalingFactors }

{---------------------------------------------------------------------------}

Procedure MultMat(Var m3, m1, m2 : Matrix);
{
  m3 := m1 * m2; all three must be unique matrices
}
Begin
  m3[1,1]:=m1[1,1]*m2[1,1]+m1[1,2]*m2[2,1]+m1[1,3]*m2[3,1]+m1[1,4]*m2[4,1];
  m3[1,2]:=m1[1,1]*m2[1,2]+m1[1,2]*m2[2,2]+m1[1,3]*m2[3,2]+m1[1,4]*m2[4,2];
  m3[1,3]:=m1[1,1]*m2[1,3]+m1[1,2]*m2[2,3]+m1[1,3]*m2[3,3]+m1[1,4]*m2[4,3];
  m3[1,4]:=m1[1,1]*m2[1,4]+m1[1,2]*m2[2,4]+m1[1,3]*m2[3,4]+m1[1,4]*m2[4,4];
  m3[2,1]:=m1[2,1]*m2[1,1]+m1[2,2]*m2[2,1]+m1[2,3]*m2[3,1]+m1[2,4]*m2[4,1];
  m3[2,2]:=m1[2,1]*m2[1,2]+m1[2,2]*m2[2,2]+m1[2,3]*m2[3,2]+m1[2,4]*m2[4,2];
  m3[2,3]:=m1[2,1]*m2[1,3]+m1[2,2]*m2[2,3]+m1[2,3]*m2[3,3]+m1[2,4]*m2[4,3];
  m3[2,4]:=m1[2,1]*m2[1,4]+m1[2,2]*m2[2,4]+m1[2,3]*m2[3,4]+m1[2,4]*m2[4,4];
  m3[3,1]:=m1[3,1]*m2[1,1]+m1[3,2]*m2[2,1]+m1[3,3]*m2[3,1]+m1[3,4]*m2[4,1];
  m3[3,2]:=m1[3,1]*m2[1,2]+m1[3,2]*m2[2,2]+m1[3,3]*m2[3,2]+m1[3,4]*m2[4,2];
  m3[3,3]:=m1[3,1]*m2[1,3]+m1[3,2]*m2[2,3]+m1[3,3]*m2[3,3]+m1[3,4]*m2[4,3];
  m3[3,4]:=m1[3,1]*m2[1,4]+m1[3,2]*m2[2,4]+m1[3,3]*m2[3,4]+m1[3,4]*m2[4,4];
  m3[4,1]:=m1[4,1]*m2[1,1]+m1[4,2]*m2[2,1]+m1[4,3]*m2[3,1]+m1[4,4]*m2[4,1];
  m3[4,2]:=m1[4,1]*m2[1,2]+m1[4,2]*m2[2,2]+m1[4,3]*m2[3,2]+m1[4,4]*m2[4,2];
  m3[4,3]:=m1[4,1]*m2[1,3]+m1[4,2]*m2[2,3]+m1[4,3]*m2[3,3]+m1[4,4]*m2[4,3];
  m3[4,4]:=m1[4,1]*m2[1,4]+m1[4,2]*m2[2,4]+m1[4,3]*m2[3,4]+m1[4,4]*m2[4,4]
End; { Procedure MultMat }

{---------------------------------------------------------------------------}

Procedure SetDSRatio(DSR : Float);
{
  Set the D/S ratio (view distance/screen size) given by DSR.
}
Begin

  If (DSR > 0) and (DSR < 10000000.0) then
    DSRatio := DSR;

End; { SetDSRatio }

{---------------------------------------------------------------------------}

Function  Transform(x,y,z : Float; Var sx,sy : Single) : Boolean;
{
  Convert a given XYZ coordinate point to an XY screen point by applying
  the current transformation matrix and other operations.  Returns False if
  the point isn't visible, True if the point is visible.
}
Var
  dsxe, dsye,
  xe, ye, ze : Float;
Begin

  { get eye coordinate }
  xe := ViewMat[1,1]*x + ViewMat[2,1]*y + ViewMat[3,1]*z + ViewMat[4,1];
  ye := ViewMat[1,2]*x + ViewMat[2,2]*y + ViewMat[3,2]*z + ViewMat[4,2];
  ze := ViewMat[1,3]*x + ViewMat[2,3]*y + ViewMat[3,3]*z + ViewMat[4,3];

  { scale and project the eye coord onto a virtual screen (no translate) }
  sx := (xe / ze) * HorzScale;
  sy := (ye / ze) * VertScale;

  { determine if the point is visible }
  dsxe := xe * DSRatio;
  dsye := ye * DSRatio;
  Transform := (dsxe >= -ze) and (dsxe <= ze) and
               (dsye >= -ze) and (dsye <= ze);

End; { Procedure Transform }

{---------------------------------------------------------------------------}

Procedure GetViewMat(Eye, Focus : Cartesian);
{
  Manufacture a perspective coordinate transformation matrix based on the
  given viewing and focus points.
}
Var
  d1,d2,              { distance from "origin" to point under consideration }
  fx,fy,fz,           { focus point }
  tr0,tr1   : Float;  { temporary real variables }
  t1,t2     : Matrix; { scratch matrices }

Const
  XRot90      : Matrix = (( 1, 0, 0, 0),  { pre-rotated about x by 90 degrees }
                          ( 0, 0,-1, 0),
                          ( 0, 1, 0, 0),
                          ( 0, 0, 0, 1));

Begin

  { translate origin to eye position and rotate about X axis by 90 degrees }
  ViewMat      := XRot90;
  ViewMat[4,1] := -Eye.X;
  ViewMat[4,2] := -Eye.Z;
  ViewMat[4,3] :=  Eye.Y;

  { compute distance components from eye to focus point }
  fx := Eye.X - Focus.X;
  fy := Eye.Y - Focus.Y;
  fz := Eye.Z - Focus.Z;

  { compute magnitude of distance between eye and focus point (d2) and the
    magnitude of the distance vector's projection in the XY plane (d1) }
  tr0 := sqr(fx) + sqr(fy);   { temp, to save 2 FP multiplies and 1 FP add }
  d1 := sqrt(tr0);
  d2 := sqrt(tr0 + sqr(fz));

  { rotate about y axis by angle dependent on focus point }
  If d1 > 1.0E-8 then begin
    t1      := IdentMatrix;
    tr0     := -fy/d1;
    tr1     :=  fx/d1;
    t1[1,1] :=  tr0;
    t1[3,3] :=  tr0;
    t1[1,3] :=  tr1;
    t1[3,1] := -tr1;
    MultMat(t2,ViewMat,t1);
    ViewMat := t2
  End; { If }

  { rotate about x axis by angle dependent on focus point }
  If d2 > 1.0E-8 then begin
    t1      := IdentMatrix;
    tr0     := d1/d2;
    tr1     := fz/d2;
    t1[2,2] :=  tr0;
    t1[3,3] :=  tr0;
    t1[2,3] :=  tr1;
    t1[3,2] := -tr1;
    MultMat(t2,ViewMat,t1);
    ViewMat := t2
  End { If }

End; { Procedure GetViewMat }


End.